Package: triangle
Title: Distribution Functions and Parameter Estimates for the Triangle
        Distribution
Version: 1.1.0
Authors@R: 
    person(given = "Rob",
           family = "Carnell",
           role = c("aut", "cre"),
           email = "bertcarnell@gmail.com",
           comment = c(ORCID = "0009-0009-0465-7564")
           )
Description: Provides the "r, q, p, and d" distribution functions for the triangle distribution.  Also includes maximum likelihood estimation of parameters.
License: GPL (>= 2)
URL: https://bertcarnell.github.io/triangle/,
        https://github.com/bertcarnell/triangle/
BugReports: https://github.com/bertcarnell/triangle/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.3)
Imports: stats4, methods, boot
Suggests: testthat, knitr, rmarkdown, MASS, Rmpfr
VignetteBuilder: knitr
Collate: 'cdfe.R' 'dtriangle.R' 'fit-plots.R' 'ltriangle.r'
        'mle-utils.R' 'mle.R' 'mom.R' 'ptriangle.r' 'qtriangle.R'
        'rtriangle.r'
NeedsCompilation: no
Packaged: 2026-04-27 01:25:01 UTC; bertc
Author: Rob Carnell [aut, cre] (ORCID: <https://orcid.org/0009-0009-0465-7564>)
Maintainer: Rob Carnell <bertcarnell@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-27 05:10:03 UTC
Built: R 4.5.2; ; 2026-04-27 09:33:00 UTC; unix

Package: triangle
Title: Distribution Functions and Parameter Estimates for the Triangle
        Distribution
Version: 1.1.0
Authors@R: 
    person(given = "Rob",
           family = "Carnell",
           role = c("aut", "cre"),
           email = "bertcarnell@gmail.com",
           comment = c(ORCID = "0009-0009-0465-7564")
           )
Description: Provides the "r, q, p, and d" distribution functions for the triangle distribution.  Also includes maximum likelihood estimation of parameters.
License: GPL (>= 2)
URL: https://bertcarnell.github.io/triangle/,
        https://github.com/bertcarnell/triangle/
BugReports: https://github.com/bertcarnell/triangle/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.3)
Imports: stats4, methods, boot
Suggests: testthat, knitr, rmarkdown, MASS, Rmpfr
VignetteBuilder: knitr
Collate: 'cdfe.R' 'dtriangle.R' 'fit-plots.R' 'ltriangle.r'
        'mle-utils.R' 'mle.R' 'mom.R' 'ptriangle.r' 'qtriangle.R'
        'rtriangle.r'
NeedsCompilation: no
Packaged: 2026-04-27 01:25:01 UTC; bertc
Author: Rob Carnell [aut, cre] (ORCID: <https://orcid.org/0009-0009-0465-7564>)
Maintainer: Rob Carnell <bertcarnell@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-27 05:10:03 UTC
Built: R 4.6.0; ; 2026-05-01 20:11:24 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(AIC,triangle_mle)
S3method(BIC,triangle_mle)
S3method(coef,triangle_mle)
S3method(confint,triangle_mle)
S3method(logLik,triangle_mle)
S3method(print,triangle_mle)
S3method(profile,triangle_mle)
S3method(summary,triangle_mle)
S3method(vcov,triangle_mle)
export(compare_triangle_fit)
export(dltriangle)
export(dtriangle)
export(pltriangle)
export(ptriangle)
export(qltriangle)
export(qqtriangle)
export(qtriangle)
export(rltriangle)
export(rtriangle)
export(standard_triangle_mle)
export(triangle_cdfe)
export(triangle_mle)
export(triangle_mom)
importClassesFrom(stats4,profile.mle)
importClassesFrom(stats4,summary.mle)
importFrom(boot,boot)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(methods,new)
importFrom(stats,BIC)
importFrom(stats,coef)
importFrom(stats,ecdf)
importFrom(stats,integrate)
importFrom(stats,nls)
importFrom(stats,nls.control)
importFrom(stats,optim)
importFrom(stats,plot.ecdf)
importFrom(stats,ppoints)
importFrom(stats,printCoefmat)
importFrom(stats,qchisq)
importFrom(stats,qqplot)
importFrom(stats,runif)
importFrom(stats,var)
importFrom(stats4,coef)
importFrom(stats4,confint)
importFrom(stats4,show)
importFrom(stats4,vcov)

#' @include dtriangle.R
#' @rdname triangle
#' @export
ptriangle <- function(q, a=0, b=1, c=(a + b)/2) {
  q1 <- length(q)
  a1 <- length(a)
  b1 <- length(b)
  c1 <- length(c)

  pTest <- function(X){
    if (any(is.na(X)))
    { # is.na is TRUE for NA, NaN, and FALSE
      if (any(is.nan(X)))
      {
        return(NaN) # to conform to punif

      } else
      {
        return(NA) # to conform to punif
      }
    } else if (X[2] > X[4] | X[3] < X[4] | (X[2] == X[4] & X[3] == X[4]))
    {
      warning("values required to be  a <= c <= b (at least one strict inequality)")
      return(NaN) # to conform to behavior of punif

    } else if (any(is.infinite(X[2:4])))
    {
      warning("a, b, c must be finite, NaN produced")
      return(NaN) # to conform to behavior of punif
    } else if (X[1] <= X[2]) {
      return(0)
    } else if (X[2] != X[4] & X[1] < X[4]) {
      return((X[1] - X[2])^2 / (X[3] - X[2]) / (X[4] - X[2]))
    } else if (X[4] != X[3] & X[1] >= X[4] & X[1] < X[3]) {
      return(1 - (X[3] - X[1])^2 / (X[3] - X[2]) / (X[3] - X[4]))
    } else if (X[1] >= X[3]) {
      return(1)
    }
  }

  k <- max(q1, a1, b1, c1)
  if (k == 1) return(pTest(c(q, a, b, c)))

  params <- matrix(nrow = k, ncol = 4)
  tryCatch(
  {
    params[,1] <- q
    params[,2] <- a
    params[,3] <- b
    params[,4] <- c
  }, error = function(X) {
    stop(paste(" -- Argument Lengths: length of q = ", q1,
                ", a = ", a1, ", b = ", b1, ", c = ", c1, " -- ", X, sep = ""))
  })

  return(apply(params, 1, pTest))
}

# Copyright 2026 Rob Carnell

test_that("qtriangle", {
  expect_equal(qtriangle(0, 1, 3, 2), 1)
  expect_equal(qtriangle(0, 0, 3, 2), 0)
  expect_equal(qtriangle(0, -1, 3, 2), -1)
  expect_equal(qtriangle(1, 1, 3, 2), 3)
  expect_equal(qtriangle(1, -3, -1, -2), -1)
  expect_equal(qtriangle(.5, 0, 1, .5), .5)
  expect_equal(qtriangle(0, 0, 0, 0), 0)
  expect_warning(expect_equal(qtriangle(-1, 0, 1, .5), NaN))
  expect_warning(expect_equal(qtriangle(2, 0, 1, .5), NaN))
  expect_warning(expect_equal(qtriangle(.5, 0, 1, 2), NaN))
  expect_warning(expect_equal(qtriangle(.5, 1, 2, .5), NaN))
  expect_equal(qtriangle(.5, NA, 2, .5), NA)
  expect_equal(qtriangle(c(0, 0, 0, 1, 1, .5), c(1, 0, -1, 1, -3, 0),
                               c(3, 3, 3, 3, -1, 1), c(2, 2, 2, 2, -2, .5)),
                     c(1, 0, -1, 3, -1, .5))
  expect_warning(expect_equal(qtriangle(c(.5, -1, NA), c(0, 0, 0), c(1, 1, 1),
                                         c(.5, .5, .5)), c(.5, NaN, NA)))
  expect_equal(qtriangle(c(0, 1, 0), 1, 2, 1.5), c(1, 2, 1))
  expect_error(qtriangle(c(0, 0, 0, 0, 0), c(0, 0), 1, .5))
  expect_equal(qtriangle(.5, NaN, 2, 1), NaN)
  expect_warning(expect_equal(qtriangle(.5, -Inf, 2, 1), NaN))
  expect_warning(expect_equal(qtriangle(.5, 0, Inf, 1), NaN))
  expect_warning(expect_equal(qtriangle(.5, -Inf, Inf, 1), NaN))
  expect_warning(expect_equal(qtriangle(.5, 0, Inf, Inf), NaN))

  # From a Bug Report, Michael.Scroggie@dse.vic.gov.au, Thursday 10/19/06
  expect_true(!all(0 == qtriangle(runif(10), 0, 1, 0)))
  expect_true(!all(5 == qtriangle(runif(10), 2, 5, 5)))
  expect_true(all(1 == qtriangle(runif(10), 1, 1, 1)))
  expect_true(all(-1 == qtriangle(runif(10), -1, -1, -1)))

  expect_equal(qtriangle(ptriangle(.5, 0, 1, 0), 0, 1, 0), 0.5)
  expect_equal(qtriangle(ptriangle(3, 2, 5, 5), 2, 5, 5), 3)
  expect_equal(qtriangle(ptriangle(5, 2, 5, 5), 2, 5, 5), 5)

  expect_equal(5, qtriangle(1, 1, 5, 5))
  expect_equal(5, qtriangle(1, 1, 5 + .Machine$double.eps, 5))
  expect_equal(5, qtriangle(1 - 2 * .Machine$double.eps, 1, 5, 5))

  # these tests both failed until the numeric tolerance was increased to 2*.Machine$double.eps
  expect_true(all(qtriangle(seq(0, 1, by = 0.1), 0, 10, 3) >= 0.0))
  expect_equal(3, qtriangle(0.0 + 0.1 + 0.1 + 0.1, 0, 10, 3))
  # this test failed until the numeric tolerance was increased to 10*.Machine$double.eps
  expect_equal(30, qtriangle(0.0 + 0.1 + 0.1 + 0.1, 0, 100, 30))
  # increased tolerance relative to the base of the triangle
  expect_equal(300, qtriangle(0.0 + 0.1 + 0.1 + 0.1, 0, 1000, 300))
  expect_equal(3000, qtriangle(0.0 + 0.1 + 0.1 + 0.1, 0, 10000, 3000))
  expect_equal(30000, qtriangle(0.0 + 0.1 + 0.1 + 0.1, 0, 100000, 30000))
  expect_equal(300000, qtriangle(0.0 + 0.1 + 0.1 + 0.1, 0, 1000000, 300000))
  expect_equal(3000000, qtriangle(0.0 + 0.1 + 0.1 + 0.1, 0, 10000000, 3000000))
})

test_that("qtriangle is consistent with runif", {
  # NaN
  expect_true(is.nan(qunif(0.5, NaN, 1)))
  expect_true(is.nan(qunif(0.5, 0, NaN)))
  expect_true(is.nan(qunif(NaN, 0, 1)))

  expect_true(is.nan(qtriangle(0.5, NaN, 1, 0.5)))
  expect_true(is.nan(qtriangle(0.5, 0, NaN, 0.5)))
  expect_true(is.nan(qtriangle(0.5, 0, 1, NaN)))
  expect_true(is.nan(qtriangle(NaN, 0, 1, 0.5)))

  # NA
  expect_true(is.na(qunif(0.5, NA, 1)))
  expect_true(is.na(qunif(0.5, 0, NA)))
  expect_true(is.na(qunif(NA, 0, 1)))

  expect_true(is.na(qtriangle(0.5, NA, 1, 0.5)))
  expect_true(is.na(qtriangle(0.5, 0, NA, 0.5)))
  expect_true(is.na(qtriangle(0.5, 0, 1, NA)))
  expect_true(is.na(qtriangle(NA, 0, 1, 0.5)))

  # out of order
  expect_warning(expect_true(all(is.nan(qunif(0.5, 1, 0)))))
  # a > c
  expect_warning(expect_true(all(is.nan(qtriangle(0.5, 5, 6, 4)))))
  # b < c
  expect_warning(expect_true(all(is.nan(qtriangle(0.5, 5, 6, 7)))))

  # Inf
  expect_warning(expect_true(is.nan(qunif(0.5, 0, Inf))))
  expect_warning(expect_true(is.nan(qunif(0.5, -Inf, 1))))
  expect_warning(expect_true(is.nan(qunif(Inf, 0, 1))))

  expect_warning(expect_true(is.nan(qtriangle(0.5, -Inf, 1, 0.5))))
  expect_warning(expect_true(is.na(qtriangle(0.5, 0, Inf, 0.5))))
  expect_warning(expect_true(is.na(qtriangle(Inf, 0, 1, 0.5))))

})

# Copyright 2026 Rob Carnell

test_that("ptriangle", {
  expect_equal(ptriangle(1, 1, 3, 2), 0)
  expect_equal(ptriangle(3, 0, 3, 2), 1)
  expect_equal(ptriangle(-1, -1, 3, 2), 0)
  expect_equal(ptriangle(.5, 0, 1, .5), .5)

  # a <= c <= b at least one strict
  expect_warning(expect_equal(ptriangle(0, 0, 0, 0), NaN))
  expect_warning(expect_equal(ptriangle(.5, 0, 1, 2), NaN))
  expect_warning(expect_equal(ptriangle(.5, 1, 2, .5), NaN))
  expect_warning(expect_equal(ptriangle(c(.5, .6), c(0, 0), c(1, 1),
                                                c(.5, 1.5)), c(.5, NaN)))
})

test_that("ptriangle is consistent with punif", {
  # NaN
  expect_true(is.nan(punif(0.5, NaN, 1)))
  expect_true(is.nan(punif(0.5, 0, NaN)))
  expect_true(is.nan(punif(NaN, 0, 1)))

  expect_true(is.nan(ptriangle(0.5, NaN, 1, 0.5)))
  expect_true(is.nan(ptriangle(0.5, 0, NaN, 0.5)))
  expect_true(is.nan(ptriangle(0.5, 0, 1, NaN)))
  expect_true(is.nan(ptriangle(NaN, 0, 1, 0.5)))

  # NA
  expect_true(is.na(punif(0.5, NA, 1)))
  expect_true(is.na(punif(0.5, 0, NA)))
  expect_true(is.na(punif(NA, 0, 1)))

  expect_true(is.na(ptriangle(0.5, NA, 1, 0.5)))
  expect_true(is.na(ptriangle(0.5, 0, NA, 0.5)))
  expect_true(is.na(ptriangle(0.5, 0, 1, NA)))
  expect_true(is.na(ptriangle(NA, 0, 1, 0.5)))

  # out of order
  expect_warning(expect_true(all(is.nan(punif(0.5, 1, 0)))))
  # a > c
  expect_warning(expect_true(all(is.nan(ptriangle(0.5, 5, 6, 4)))))
  # b < c
  expect_warning(expect_true(all(is.nan(qtriangle(0.5, 5, 6, 7)))))

  # Inf
  expect_warning(expect_true(is.nan(punif(0.5, 0, Inf))))
  expect_warning(expect_true(is.nan(punif(0.5, -Inf, 1))))

  expect_warning(expect_true(is.nan(ptriangle(0.5, -Inf, 1, 0.5))))
  expect_warning(expect_true(is.na(ptriangle(0.5, 0, Inf, 0.5))))

  # multi-parameters
  expect_equal(punif(c(0, 4, -1, 2), 0, 4), c(0, 1, 0, 0.5))
  expect_equal(ptriangle(c(1, 3, -1, .5), c(1, 0, -1, 0), c(3, 3, 3, 1),
                         c(2, 2, 2, .5)), c(0, 1, 0, .5))
})

test_that("ptriangle is not consistent with punif", {
  # punif does recylcing here
  # punif(4, c(1,2), c(2,3,4)) will return 3 numbers
  expect_error(ptriangle(c(1,2,3), c(1,2,3), c(4,5), c(1,2,3)))
})

# Copyright Robert Carnell 2022

#' Log of the maximization function.  The natural log of equation 1.43
#'
#' @references Samuel Kotz and Johan Rene van Dorp. Beyond Beta \doi{10.1142/5720}
#'
#' @noRd
#'
#' @param z the set of order statistics (sorted sample)
#' @param a the minimum support of the triangle distribution \code{a < b}, \code{a <= min(z)}
#' @param b the maximum support of the triangle distribution \code{b >= max(z)}
#' @param r the order statistic at which M is to be calculated \code{1 <= r <= length(z)}
#' @param debug if \code{TRUE} then \code{logM} will check the input parameters
#'
#' @return The \code{log(M)} from equation 1.43
#'
#' @examples
#' set.seed(103)
#' logM(rtriangle(10, 0, 1, 0.5), 0, 1, 3)
logM <- function(z, a, b, r, debug = FALSE)
{
  s <- length(z)
  if (debug)
  {
    if (length(r) != 1) stop("r must be a scalar")
    if (length(a) != 1) stop("a must be a scalar")
    if (length(b) != 1) stop("b must be a scalar")
    if (r > s | r < 1) stop("r must be on [1, length(z)]")
    if (a >= b) stop("a < b")
    if (a > min(z)) stop("a <= min(z)")
    if (b < max(z)) stop("b >= max(z)")
    if (!(all(order(z) == 1:s))) stop("z must be sorted")
  }
  if (r > 1 & r < s)
  {
    #return(prod((z[1:(rk-1)] - ak) / (z[rk] - ak)) * prod((bk - z[(rk+1):s]) / (bk - z[rk])))
    return(sum(log(z[1:(r-1)] - a)) - (r-1)*log(z[r] - a) + sum(log(b - z[(r+1):s])) - (s-r)*log(b - z[r]))
  } else if (r == 1)
  {
    #return(prod((bk - z[(rk+1):s]) / (bk - z[rk])))
    return(sum(log(b - z[(r+1):s])) - (s-r)*log(b - z[r]))
  } else if (r == s)
  {
    #return(prod((z[1:(rk-1)] - ak) / (z[rk] - ak)))
    return(sum(log(z[1:(r-1)] - a)) - (s-1)*log(z[r] - a))
  }
}

#' The order statistic which is the estimate of the mode of the triangle distribution
#' for a given \code{a} and \code{b} from equation 1.42
#'
#' @references Samuel Kotz and Johan Rene van Dorp. Beyond Beta \doi{10.1142/5720}
#'
#' @noRd
#'
#' @param z the set of order statistics (sorted sample)
#' @param a the minimum support of the triangle distribution \code{a < b}, \code{a <= min(z)}
#' @param b the maximum support of the triangle distribution \code{b >= max(z)}
#' @param debug if \code{TRUE} then the function will check the input parameters
#'
#' @return the order statistic number
#'
#' @examples
#' set.seed(40039)
#' rhat(rtriangle(10, 0, 1, 0.5), 0, 1)
rhat <- function(z, a, b, debug = FALSE)
{
  s <- length(z)
  if (debug)
  {
    if (length(a) != 1) stop("a must be a scalar")
    if (length(b) != 1) stop("b must be a scalar")
    if (a >= b) stop("a < b")
    if (a > min(z)) stop("a <= min(z)")
    if (b < max(z)) stop("b >= max(z)")
    if (!(all(order(z) == 1:s))) stop("z must be sorted")
  }
  res <- sapply(1:s, function(r) logM(z, a, b, r, debug))
  return(which.max(res))
}

#' Maximum likelihood estimate of c given a and b
#'
#' @noRd
#'
#' @param x sample from a triangle distribution
#' @param a the minimum support of the triangle distribution \code{a < b}, \code{a <= min(z)}
#' @param b the maximum support of the triangle distribution \code{b >= max(z)}
#' @param debug if \code{TRUE} then the function will check the input parameters
#'
#' @return a list containing the estimate of \code{c} and \code{r}
#'
#' @examples
#' xtest <- c(0.1, 0.25, 0.3, 0.4, 0.45, 0.6, 0.75, 0.8)
#' triangle_mle_c_given_ab(xtest, 0, 1)$c_hat
triangle_mle_c_given_ab <- function(x, a, b, debug = FALSE)
{
  if (debug)
  {
    if (length(a) != 1) stop("a must be a scalar")
    if (length(b) != 1) stop("b must be a scalar")
    if (a >= b) stop("a < b")
    if (a > min(x)) stop("a must be <= min(x)")
    if (b < max(x)) stop("b must be >= max(x)")
  }
  sx <- sort(x)
  r <- rhat(sx, a, b, debug)
  c_hat <- sx[r]
  if (debug)
  {
    stopifnot(a <= c_hat & c_hat <= b)
  }
  return(list(c_hat = c_hat, r_hat = r))
}

#' Negative log likelihood of a sample given a, b, and c
#'
#' @noRd
#'
#' @param x sample from a triangle distribution
#' @param a the minimum support of the triangle distribution \code{a < b}, \code{a <= min(z)}
#' @param b the maximum support of the triangle distribution \code{b >= max(z)}
#' @param c the mode of the triangle distribution
#' @param debug if \code{TRUE} then the function will check the input parameters
#'
#' @return the negative log likelihood
#'
#' @examples
#' xtest <- c(0.1, 0.25, 0.3, 0.4, 0.45, 0.6, 0.75, 0.8)
#' nLL_triangle(xtest, 0, 1, 0.3)
nLL_triangle <- function(x, a, b, c, debug = FALSE)
{
  n <- length(x)
  if (debug)
  {
    if (length(a) != 1) stop("a must be a scalar")
    if (length(b) != 1) stop("b must be a scalar")
    if (a >= b) stop("a < b")
    if (a > min(x)) stop(paste0("a (", a, ") <= min(x) (", min(x), ")"))
    if (b < max(x)) stop("b >= max(x)")
  }
  # a_hat = min(x) when a = c < b
  if (a == min(x) & a == c) {
    ret <- -n*log(2) + n*log(b-a) + n*log(b-c) - sum(log(b - x))
  } else if (b == max(x) & b == c) {
    ret <- -n*log(2) + n*log(b-a) + n*log(c-a) - sum(log(x - a))
  } else if (a == min(x) | b == max(x)) {
    ret <- Inf
  } else {
    ind1 <- which(x < c)
    ind2 <- which(x >= c)
    n1 <- length(ind1)
    n2 <- length(ind2)
    if (n1 > 0 & n2 > 0)
    {
      ret <- -n*log(2) + n*log(b-a) + n1*log(c-a) + n2*log(b-c) - sum(log(x[ind1] - a)) - sum(log(b - x[ind2]))
    } else if (n1 == 0)
    {
      ret <- -n*log(2) + n*log(b-a) + n*log(b-c) - sum(log(b - x[ind2]))
    } else if (n2 == 0)
    {
      ret <- -n*log(2) + n*log(b-a) + n*log(c-a) - sum(log(x[ind1] - a))
    }
  }
  return(ret)
}

#' Gradient of the Negative log likelihood of a sample given a fixed c
#'
#' @noRd
#'
#' @param x sample from a triangle distribution
#' @param a the minimum support of the triangle distribution \code{a < b}, \code{a <= min(z)}
#' @param b the maximum support of the triangle distribution \code{b >= max(z)}
#' @param c the mode of the triangle distribution
#' @param debug if \code{TRUE} then the function will check the input parameters
#'
#' @return the gradient of the negative log likelihood with components for \code{d/da} and \code{d/db}
#'
#' @examples
#' xtest <- c(0.1, 0.25, 0.3, 0.4, 0.45, 0.6, 0.75, 0.8)
#' gradient_nLL_triangle_given_c(xtest, 0, 1, 0.3)
gradient_nLL_triangle_given_c <- function(x, a, b, c, debug = FALSE)
{
  if (debug)
  {
    if (length(a) != 1) stop("a must be a scalar")
    if (length(b) != 1) stop("b must be a scalar")
    if (a >= b) stop("a < b")
    if (a > min(x)) stop(paste0("a (", a, ") <= min(x) (", min(x), ")"))
    if (b < max(x)) stop("b >= max(x)")
  }
  n <- length(x)
  ind1 <- which(x < c)
  ind2 <- which(x >= c)
  n1 <- length(ind1)
  n2 <- length(ind2)
  if (n1 > 0 & n2 > 0)
  {
    return(c(-n/(b - a) - ifelse(c > a, n1/(c - a), 0) + sum(ifelse(x[ind1] > a, 1/(x[ind1] - a), 0)),
             n/(b - a) + ifelse(b > c, n2/(b - c), 0) - sum(ifelse(x[ind2] < b, 1/(b - x[ind2]), 0))))
  } else if (n1 == 0)
  {
    return(c(-n/(b - a),
             n/(b - a) + ifelse(b > c, n2/(b - c), 0) - sum(ifelse(x[ind2] < b, 1/(b - x[ind2]), 0))))
  } else if (n2 == 0)
  {
    return(c(-n/(b - a) - ifelse(c > a, n1/(c - a), 0) + sum(ifelse(x[ind1] > a, 1/(x[ind1] - a), 0)),
             n/(b - a)))
  } else
  {
    stop("Unexpected result in gradient")
  }
}

#' Hessian of the Negative log likelihood of a sample given a fixed c
#'
#' @noRd
#'
#' @param x sample from a triangle distribution
#' @param a the minimum support of the triangle distribution \code{a < b}, \code{a <= min(z)}
#' @param b the maximum support of the triangle distribution \code{b >= max(z)}
#' @param c the mode of the triangle distribution
#' @param debug if \code{TRUE} then the function will check the input parameters
#'
#' @return the hessian of the negative log likelihood with respect to a and b
#'
#' @examples
#' xtest <- c(0.1, 0.25, 0.3, 0.4, 0.45, 0.6, 0.75, 0.8)
#' hessian_nLL_triangle_given_c(xtest, 0, 1, 0.3)
hessian_nLL_triangle_given_c <- function(x, a, b, c, debug = FALSE)
{
  if (debug)
  {
    if (a >= b) stop("a must be less than b")
  }
  n <- length(x)
  ind1 <- which(x < c)
  ind2 <- which(x >= c)
  n1 <- length(ind1)
  n2 <- length(ind2)
  dadb <- n / (b - a)^2
  if (n1 > 0 & n2 > 0)
  {
    da2 <- -n/(b - a)^2 - ifelse(c > a, n1/(c - a)^2, 0) + sum(ifelse(x[ind1] > a, 1/(x[ind1] - a)^2, 0))
    db2 <- -n/(b - a)^2 - ifelse(b > c, n2/(b - c)^2, 0) + sum(ifelse(x[ind2] < b, 1/(b - x[ind2])^2, 0))
  } else if (n1 == 0)
  {
    da2 <- -n/(b - a)^2
    db2 <- -n/(b - a)^2 - ifelse(b > c, n2/(b - c)^2, 0) + sum(ifelse(x[ind2] < b, 1/(b - x[ind2])^2, 0))
  } else if (n2 == 0)
  {
    da2 <- -n/(b - a)^2 - ifelse(c > a, n1/(c - a)^2, 0) + sum(ifelse(x[ind1] > a, 1/(x[ind1] - a)^2, 0))
    db2 <- -n/(b - a)^2
  } else
  {
    stop("Unexpected result in hessian")
  }
  return(matrix(c(da2, dadb, dadb, db2), nrow = 2, dimnames = list(c("a","b"), c("a","b"))))
}


#' Maximum likelihood estimate of a and b given a fixed c
#'
#' @noRd
#'
#' @param x sample from a triangle distribution
#' @param c the mode of the triangle distribution
#' @param debug if \code{TRUE} then the function will check the input parameters
#' @param start starting values for a and b
#' @param lower lower bounds for a and b
#' @param upper upper bounds for a and b
#'
#' @importFrom stats optim
#'
#' @return a list containing a, b, the results of stats::optim, and the analytic hessian
#'
#' @examples
#' xtest <- c(0.1, 0.25, 0.3, 0.4, 0.45, 0.6, 0.75, 0.8)
#' triangle_mle_ab_given_c(xtest, 0.3)
triangle_mle_ab_given_c <- function(x, c, debug = FALSE, start = NA, lower = NA, upper = NA)
{
  if (debug)
  {
    if (length(c) != 1) stop("c must be a scalar")
    if (c > max(x)) stop("c <= min(x)")
    if (c < min(x)) stop("c >= max(x)")
  }
  # x <- triangle::rtriangle(100, 0.001, 0.005, 0.004)
  nLL <- function(p, x, c, debug)
  {
    nLL_triangle(x, p[1], p[2], c, debug)
  }

  g_nLL <- function(p, x, c, debug)
  {
    gradient_nLL_triangle_given_c(x, p[1], p[2], c, debug)
  }

  minx <- min(x, na.rm = TRUE)
  maxx <- max(x, na.rm = TRUE)
  rangex <- maxx - minx
  if (any(is.na(start)))
  {
    # start at method of moments estimates, but a little wider
    start <- c(minx - 0.05*rangex,
               maxx + 0.05*rangex)
  }
  if (any(is.na(lower)) | any(is.na(upper)))
  {
    lower <- c(minx - 2*rangex, # lower end of a can be low
               maxx + abs(.Machine$double.eps*maxx)) # lower end of b can be just above max(x)
    upper <- c(minx - abs(.Machine$double.eps*minx),
               maxx + 2*rangex)
  }
  mle_ab <- stats::optim(par = start, fn = nLL, gr = g_nLL, x = x, c = c, debug = debug,
                  method = "L-BFGS-B", lower = lower, upper = upper,
                  hessian = FALSE)
  return(list(a = mle_ab$par[1],
              b = mle_ab$par[2],
              optim = mle_ab,
              hessian_ab = hessian_nLL_triangle_given_c(x, mle_ab$par[1], mle_ab$par[2], c, debug)))
}

#' Density of the rth order statistic
#'
#' @noRd
#'
#' @param x The value of the rth order statistic
#' @param n number of order statistics \code{n > 0}
#' @param r the order statistic number \code{0 < r <= n}
#' @param a the minimum support of the triangle distribution \code{a < b}
#' @param b the maximum support of the triangle distribution
#' @param c the mode of the triangle distribution
#'
#' @return the density
#'
#' @examples
#' integrate(f_rth_order_stat, lower = 0, upper = 1, n = 10, r = 3, a = 0, b = 1, c = 0.5)
f_rth_order_stat <- function(x, n, r, a, b, c)
{
  ret <- exp(log(r) + lchoose(n, r) + log(dtriangle(x, a, b, c)) + (r-1) * log(ptriangle(x, a, b, c)) + (n-r) * log(1 - ptriangle(x, a, b, c)))
  # need to trap case when ptriangle returns 1 and r = n which causes an NaN (o * Inf)
  if (any(r == n))
  {
    ind <- which(r == n & x >= b)
    ret[ind] <- 0
  }
  return(ret)
}

#' Expected value of the rth order statistic
#'
#' @noRd
#'
#' @param n number of order statistics
#' @param r the order statistic number
#' @param a the minimum support of the triangle distribution \code{a < b}, \code{a <= min(z)}
#' @param b the maximum support of the triangle distribution \code{b >= max(z)}
#' @param c the mode of the triangle distribution
#'
#' @return the expected value
#'
#' @examples
#' mean_rth_order_stat(10, 5, 0, 1, 0.5)
mean_rth_order_stat <- function(n, r, a, b, c)
{
  return(mean_rth_order_stat_numeric(n, r, a, b, c))
}

#' Expected value of the rth order statistic using base precision
#'
#' @noRd
#'
#' @param n number of order statistics
#' @param r the order statistic number
#' @param a the minimum support of the triangle distribution \code{a < b}, \code{a <= min(z)}
#' @param b the maximum support of the triangle distribution \code{b >= max(z)}
#' @param c the mode of the triangle distribution
#'
#' @return the expected value
#'
#' @examples
#' mean_rth_order_stat_base(10, 5, 0, 1, 0.5)
mean_rth_order_stat_base <- function(n, r, a, b, c)
{
  coefs1 <- sapply(0:(n-r), function(k) {
    choose(n-r, k) * (b-a)^(k-n) * (c-a)^(n-k) * (-1)^(n-r-k) *
      (c/(n-k) - (c-a)/(n-k)/(2*n-2*k+1))
  })

  coefs2 <- sapply(0:(r-1), function(k) {
    choose(r-1, k) * (b-a)^(k-n) * (c-b)^(n-k) *
      (-c/(n-k) + (c-b)/(n-k)/(2*n-2*k+1))
  })

  return(ifelse(c > a, r*choose(n, r)*sum(coefs1), 0) +
           ifelse(c < b, r*choose(n, r)*(-1)^(n-r)*sum(coefs2), 0))
}

#' Expected value of the rth order statistic using multiple precision
#'
#' @noRd
#'
#' @param n number of order statistics
#' @param r the order statistic number
#' @param a the minimum support of the triangle distribution \code{a < b}, \code{a <= min(z)}
#' @param b the maximum support of the triangle distribution \code{b >= max(z)}
#' @param c the mode of the triangle distribution
#'
#' @return the expected value
#'
#' @examples
#' if (requireNamespace("Rmpfr", quietly = TRUE)) mean_rth_order_stat_rmpfr(10, 5, 0, 1, 0.5)
mean_rth_order_stat_rmpfr <- function(n, r, a, b, c)
{
  if (!requireNamespace("Rmpfr", quietly = TRUE))
    stop("Rmpfr is required for this function")

  calc_prec <- 2^8
  n <- as.integer(n)
  r <- as.integer(r)
  a <- Rmpfr::mpfr(a, calc_prec)
  b <- Rmpfr::mpfr(b, calc_prec)
  c <- Rmpfr::mpfr(c, calc_prec)

  f1 <- function(k) {
    (b-a)^(k-n) * (c-a)^(n-k) * (c/(n-k) - (c-a)/(n-k)/(2*n-2*k+1))
  }
  coefs1 <- Rmpfr::sumBinomMpfr(n-r, f1, n0 = 0, alternating = TRUE, precBits = calc_prec)

  f2 <- function(k) {
    (b-a)^(k-n) * (c-b)^(n-k) * (-c/(n-k) + (c-b)/(n-k)/(2*n-2*k+1))
  }
  coefs2 <- Rmpfr::sumBinomMpfr(r-1, f2, n0 = 0, alternating = FALSE, precBits = calc_prec)

  if (c == a) {
    tot <- r * Rmpfr::chooseMpfr(n, r) * Rmpfr::mpfr((-1)^(n-r), calc_prec) * coefs2
  } else if (c == b) {
    tot <- r * Rmpfr::chooseMpfr(n, r) * coefs1
  } else {
    tot <- r * Rmpfr::chooseMpfr(n, r) *
      (coefs1 + Rmpfr::mpfr((-1)^(n-r), 2^8) * coefs2)
  }

  return(as.numeric(tot))
}

#' Expected value of the rth order statistic using numeric integration
#'
#' @noRd
#'
#' @param n number of order statistics
#' @param r the order statistic number
#' @param a the minimum support of the triangle distribution \code{a < b}, \code{a <= min(z)}
#' @param b the maximum support of the triangle distribution \code{b >= max(z)}
#' @param c the mode of the triangle distribution
#'
#' @return the expected value
#'
#' @importFrom stats integrate
#' @examples
#' mean_rth_order_stat_numeric(10, 5, 0, 1, 0.5)
mean_rth_order_stat_numeric <- function(n, r, a, b, c)
{
  integrand <- function(x, n, r, a, b, c) {x * f_rth_order_stat(x, n, r, a, b, c)}
  stats::integrate(integrand, lower = a, upper = b, n = n, r = r, a = a, b = b, c = c)$value
}

#' Variance of the rth order statistic
#'
#' @noRd
#'
#' @param n number of order statistics
#' @param r the order statistic number
#' @param a the minimum support of the triangle distribution \code{a < b}, \code{a <= min(z)}
#' @param b the maximum support of the triangle distribution \code{b >= max(z)}
#' @param c the mode of the triangle distribution
#'
#' @return the variance
#'
#' @examples
#' variance_rth_order_stat(10, 5, 0, 1, 0.5)
variance_rth_order_stat <- function(n, r, a, b, c)
{
  variance_rth_order_stat_numeric(n, r, a, b, c)
}

#' Variance of the rth order statistic using base summation
#'
#' @noRd
#'
#' @param n number of order statistics
#' @param r the order statistic number
#' @param a the minimum support of the triangle distribution \code{a < b}, \code{a <= min(z)}
#' @param b the maximum support of the triangle distribution \code{b >= max(z)}
#' @param c the mode of the triangle distribution
#'
#' @return the variance
#'
#' @examples
#' variance_rth_order_stat_base(10, 5, 0, 1, 0.5)
variance_rth_order_stat_base <- function(n, r, a, b, c)
{
  coefs1 <- sapply(0:(n-r), function(k) {
    choose(n-r, k) * (b-a)^(k-n) * (c-a)^(n-k) * (-1)^(n-r-k) / (n-k) *
      (c^2 - 2*c*(c-a)/(2*n-2*k+1) + 2*(c-a)^2/(2*n-2*k+1)/(2*n-2*k+2))
  })
  coefs2 <- sapply(0:(r-1), function(k) {
    choose(r-1, k) * (b-a)^(k-n) * (c-b)^(n-k)/(n-k) *
      (-c^2 + 2*c*(c-b)/(2*n-2*k+1) - 2*(c-b)^2/(2*n-2*k+1)/(2*n-2*k+2))
  })
  m <- mean_rth_order_stat_base(n, r, a, b, c)
  return(ifelse(c > a, r*choose(n, r)*sum(coefs1), 0) +
           ifelse(c < b, r*choose(n, r)*(-1)^(n-r)*sum(coefs2), 0) - m^2)
}

#' Variance of the rth order statistic using multi-precision computation
#'
#' @noRd
#'
#' @param n number of order statistics
#' @param r the order statistic number
#' @param a the minimum support of the triangle distribution \code{a < b}, \code{a <= min(z)}
#' @param b the maximum support of the triangle distribution \code{b >= max(z)}
#' @param c the mode of the triangle distribution
#'
#' @return the variance
#'
#' @examples
#' if (requireNamespace("Rmpfr")) variance_rth_order_stat_rmpfr(10, 5, 0, 1, 0.5)
variance_rth_order_stat_rmpfr <- function(n, r, a, b, c)
{
  if (!requireNamespace("Rmpfr", quietly = TRUE))
    stop("Rmpfr is required for this function")

  calc_prec <- 2^8
  n <- as.integer(n)
  r <- as.integer(r)
  a <- Rmpfr::mpfr(a, calc_prec)
  b <- Rmpfr::mpfr(b, calc_prec)
  c <- Rmpfr::mpfr(c, calc_prec)

  f1 <- function(k) {
    (b-a)^(k-n) * (c-a)^(n-k) / (n-k) * (c^2 - 2*c*(c-a)/(2*n-2*k+1) + 2*(c-a)^2/(2*n-2*k+1)/(2*n-2*k+2))
  }
  coefs1 <- Rmpfr::sumBinomMpfr(n-r, f1, n0 = 0, alternating = TRUE, precBits = calc_prec)

  f2 <- function(k) {
    (b-a)^(k-n) * (c-b)^(n-k) / (n-k) * (-c^2 + 2*c*(c-b)/(2*n-2*k+1) - 2*(c-b)^2/(2*n-2*k+1)/(2*n-2*k+2))
  }
  coefs2 <- Rmpfr::sumBinomMpfr(r-1, f2, n0 = 0, alternating = FALSE, precBits = calc_prec)

  m <- mean_rth_order_stat_rmpfr(n, r, a, b, c)

  if (c == a) {
    tot <- r * Rmpfr::chooseMpfr(n, r) * Rmpfr::mpfr((-1)^(n-r), calc_prec) * coefs2 - m^2
  } else if (c == b) {
    tot <- r * Rmpfr::chooseMpfr(n, r) * coefs1 - m^2
  } else {
    tot <- r * Rmpfr::chooseMpfr(n, r) *
      (coefs1 + Rmpfr::mpfr((-1)^(n-r), 2^8) * coefs2) - m^2
  }

  return(as.numeric(tot))
}

#' Variance of the rth order statistic using numeric integration
#'
#' @noRd
#'
#' @param n number of order statistics
#' @param r the order statistic number
#' @param a the minimum support of the triangle distribution \code{a < b}, \code{a <= min(z)}
#' @param b the maximum support of the triangle distribution \code{b >= max(z)}
#' @param c the mode of the triangle distribution
#'
#' @return the variance
#'
#' @importFrom stats integrate
#' @examples
#' variance_rth_order_stat_numeric(10, 5, 0, 1, 0.5)
variance_rth_order_stat_numeric <- function(n, r, a, b, c)
{
  integrand <- function(x, n, r, a, b, c) {x * x * f_rth_order_stat(x, n, r, a, b, c)}
  E_x2 <- stats::integrate(integrand, lower = a, upper = b, n = n, r = r, a = a, b = b, c = c)
  E_x <- mean_rth_order_stat_numeric(n, r, a, b, c)
  E_x2$value - E_x * E_x
}

iterative_triangle_mle <- function(x, debug = FALSE, maxiter = 100)
{
  sx <- sort(x)
  n <- length(x)
  # if c = sx[1] then a will optimize to c = a = sx[1]
  # if c = sx[n] then b will optimize to c = b = sx[n]
  # start at MOM estimate
  mom <- triangle_mom(x, na.rm = TRUE, type = 2)
  # find x closest to mom["c"]
  rhat1 <- which.min(abs(sx - mom["c"]))
  mle_c0 <- sx[rhat1]
  if (rhat1 == 1) {
    mle_ab <- triangle_mle_ab_given_c(x, c = mle_c0, debug = debug, start = c(mle_c0, mom["b"]))
  } else if (rhat1 == n) {
    mle_ab <- triangle_mle_ab_given_c(x, c = mle_c0, debug = debug, start = c(mom["a"], mle_c0))
  } else {
    mle_ab <- triangle_mle_ab_given_c(x, c = mle_c0, debug = debug, start = c(mom["a"], mom["b"]))
  }

  # then determine c given a and b
  mle_c1 <- triangle_mle_c_given_ab(x, mle_ab$a, mle_ab$b, debug = debug)

  # then optimize a and b given c
  mle_ab <- triangle_mle_ab_given_c(x, c = mle_c1$c_hat, debug = debug, start = c(mle_ab$a, mle_ab$b))

  # then check to see if c has changed
  mle_c2 <- triangle_mle_c_given_ab(x, mle_ab$a, mle_ab$b, debug = debug)

  # as long as the estimate of c is changing, keep going up to max iter
  count <- 1
  while (mle_c1$c_hat != mle_c2$c_hat & count < maxiter)
  {
    mle_c1 <- mle_c2

    mle_ab <- triangle_mle_ab_given_c(x, c = mle_c1$c_hat, debug = debug, start = c(mle_ab$a, mle_ab$b))

    mle_c2 <- triangle_mle_c_given_ab(x, mle_ab$a, mle_ab$b, debug = debug)

    count <- count + 1
  }

  if (count >= maxiter)
  {
    warning("Maximum iterations reached in triangle_mle without convergence on c")
  } else if (debug)
  {
    cat("\n", count, " iterations reached\n")
  }
  return(list(a=unname(mle_ab$a), b=unname(mle_ab$b),
              c=mle_c2$c_hat, r=mle_c2$r_hat,
              hessian_ab=mle_ab$hessian_ab, optim=mle_ab$optim))
}

enumerative_triangle_mle <- function(x, debug = FALSE)
{
  #x <- xtest_small
  sx <- sort(x)
  n <- length(x)
  mom <- triangle_mom(x, na.rm = TRUE, type = 2)
  res <- sapply(sx, function(ci) {
    mle_ab <- triangle_mle_ab_given_c(x, c = ci, debug = FALSE,
                                      start = c(min(mom["a"], sx[1]),
                                                max(mom["b"], sx[n])))
    nLL <- nLL_triangle(x, mle_ab$a, mle_ab$b, ci, debug = FALSE)
    return(list(a=mle_ab$a, b=mle_ab$b, c=ci, nLL = nLL,
                hessian_ab = mle_ab$hessian_ab,
                optim = mle_ab$optim))
  })
  ind <- which.min(res["nLL",])

  return(list(a=res[,ind]$a, b=res[,ind]$b,
              c=res[,ind]$c, r=ind,
              hessian_ab=res[,ind]$hessian_ab,
              optim=res[,ind]$optim))
}

#' Maximum likelihood estimate of the triangle distribution parameters
#'
#' @references Samuel Kotz and Johan Rene van Dorp. Beyond Beta \doi{10.1142/5720}
#'
#' @param x sample from a triangle distribution
#' @param debug if \code{TRUE} then the function will check the input parameters and print calculation information
#' @param maxiter the maximum number of cycles of optimization between maximizing \code{a} and \code{b} given \code{c}
#' and maximizing \code{c} given \code{a} and \code{b}
#' @param boot_var should the variance be computed with a bootstrap sample?
#' @param boot_rep The number of bootstrap replications
#'
#' @return an object of S3 class \code{triangle_mle} containing a list with the call, coefficients,
#' variance co-variance matrix, minimum negative log likelihood, details of the optimization
#' number of observations, and the sample
#' @export
#'
#' @importFrom boot boot
#' @importFrom stats var
#'
#' @examples
#' xtest <- c(0.1, 0.25, 0.3, 0.4, 0.45, 0.6, 0.75, 0.8)
#' triangle_mle(xtest)
#'
#' xtest <- rtriangle(20, 1, 5, 3.5)
#' triangle_mle(xtest)
triangle_mle <- function(x, debug = FALSE, maxiter = 100, boot_var = FALSE, boot_rep = 500)
{
  # x <- triangle::rtriangle(100, 0, 1, 0.5)
  # debug <- TRUE
  # maxiter <- 100
  # boot_var <- TRUE
  # boot_rep <- 100
  my_call <- match.call()
  minx <- min(x, na.rm = TRUE)
  maxx <- max(x, na.rm = TRUE)
  rangex <- maxx - minx

  if (length(x) <= 20) {
    mle <- enumerative_triangle_mle(x, debug = debug)
  } else {
    mle <- iterative_triangle_mle(x, debug = debug, maxiter = maxiter)
  }

  # check to see if there is an NA in the coefficients
  if (any(is.na(mle))) {
    print(x)
    print(minx)
    print(maxx)
    print(rangex)
    stop("NA coefficient")
  }

  var_chat <- variance_rth_order_stat(length(x), mle$r, mle$a, mle$b, mle$c)

  if (any(var_chat < 0) | debug) {
    if (any(var_chat < 0)) {
      cat("\nNegative Varince in c hat\n")
    }
    cat("n=", length(x), "\n")
    cat("r=", mle$r, "\n")
    cat("a=", mle$a, "\n")
    cat("b=", mle$b, "\n")
    cat("c=", mle$c, "\n")
  }

  if (boot_var) {
    b <- boot::boot(x, statistic = function(d, i) {
      mle <- iterative_triangle_mle(d[i], debug = FALSE, maxiter = maxiter)
      return(c(mle$a, mle$b, mle$c))
    }, R = boot_rep)

    vcov_theta <- stats::var(b$t)
    dimnames(vcov_theta) <- list(c("a", "b", "c"), c("a", "b", "c"))
  } else {
    hess <- hessian_nLL_triangle_given_c(x, mle$a, mle$b, mle$c, debug = debug)
    # or hess <- mle$hessian_ab
    var_hess <- NA
    try({
      var_hess <- solve(hess)
    }, silent = TRUE)
    if (inherits(var_hess, "matrix") & all(diag(var_hess) >= 0)) {
      vcov_theta <- rbind(cbind(var_hess, c(0, 0)), c(0, 0, var_chat))
    } else {
      vcov_theta <- matrix(NA, nrow = 3, ncol = 3)
    }
    dimnames(vcov_theta) <- list(c("a", "b", "c"), c("a", "b", "c"))
  }

  structure(list(call = my_call,
                 coef = c(a = mle$a, b = mle$b, c = mle$c),
                 vcov = vcov_theta,
                 min = mle$optim$value,
                 details = mle$optim,
                 nobs = length(x),
                 x = x),
            class = "triangle_mle")
}

#' Maximum likelihood estimate of the standard triangle distribution mode
#'
#' @references Samuel Kotz and Johan Rene van Dorp. Beyond Beta \doi{10.1142/5720}
#'
#' @param x sample from a triangle distribution
#' @param debug if \code{TRUE} then the function will check the input parameters and print calculation information
#'
#' @return an object of S3 class \code{triangle_mle} containing a list with the call, coefficients,
#' variance co-variance matrix, minimum negative log likelihood,
#' number of observations, and the sample
#' @export
#'
#' @examples
#' xtest <- c(0.1, 0.25, 0.3, 0.4, 0.45, 0.6, 0.75, 0.8)
#' standard_triangle_mle(xtest)
#'
#' xtest <- rtriangle(20, 0, 1, 0.63)
#' standard_triangle_mle(xtest)
standard_triangle_mle <- function(x, debug = FALSE)
{
  my_call <- match.call()
  minx <- min(x, na.rm = TRUE)
  maxx <- max(x, na.rm = TRUE)

  if (minx < 0 | maxx > 1) stop("standard triangle requires all samples be between [0, 1]")

  # determine c at a = min(x), b = max(x)
  mle_c1 <- triangle_mle_c_given_ab(x, 0, 1, debug = debug)

  var_chat <- variance_rth_order_stat(length(x), mle_c1$r_hat, 0, 1, mle_c1$c_hat)

  structure(list(call = my_call,
                 coef = c(a = 0, b = 1, c = mle_c1$c_hat),
                 vcov = matrix(c(rep(0, 8), var_chat), nrow = 3, ncol = 3),
                 min = nLL_triangle(x, 0, 1, mle_c1$c_hat, debug),
                 details = NA,
                 nobs = length(x),
                 x = x),
            class = "triangle_mle")
}

# Copyright 2026 Rob Carnell

test_that("logM works", {
  expect_true(abs(exp(logM(xtest_small, 0, 1, 1)) - 0.007) < 0.001)
  expect_true(abs(exp(logM(xtest_small, 0, 1, 2)) - 0.010) < 0.001)
  expect_true(abs(exp(logM(xtest_small, 0, 1, 3)) - 0.011) < 0.001)
  expect_true(abs(exp(logM(xtest_small, 0, 1, 4)) - 0.010) < 0.001)
  expect_true(abs(exp(logM(xtest_small, 0, 1, 5)) - 0.009) < 0.001)
  expect_true(abs(exp(logM(xtest_small, 0, 1, 6)) - 0.005) < 0.001)
  expect_true(abs(exp(logM(xtest_small, 0, 1, 7)) - 0.004) < 0.001)
  expect_true(abs(exp(logM(xtest_small, 0, 1, 8, debug = TRUE)) - 0.003) < 0.001)
})

test_that("rhat works", {
  expect_equal(3, rhat(xtest_small, 0, 1))
  expect_equal(3, rhat(xtest_small, 0, 1, debug = TRUE))
})

test_that("triangle_mle_c_given_ab works", {
  expect_equal(0.3, triangle_mle_c_given_ab(xtest_small, 0, 1)$c_hat)

  expect_equal(0.3, triangle_mle_c_given_ab(xtest_small, 0, 1, debug = TRUE)$c_hat)
})

test_that("nLL_triangle works", {
  expect_equal(-sum(log(dtriangle(xtest_small, 0, 1, 0.3))),
               nLL_triangle(xtest_small, 0, 1, 0.3),
               tolerance = 1E-5)

  expect_equal(-log(1/2), nLL_triangle(1, 0, 2, 2, debug = TRUE))
  expect_equal(-log(1/2), nLL_triangle(1, 0, 2, 0))
  expect_equal(-log(1), nLL_triangle(1, 0, 2, 1))

  # when a = c < b, then the mle of a is min(x)
  expect_equal(-log(dtriangle(0, 0, 2, 0)), nLL_triangle(0, 0, 2, 0))
  expect_equal(-log(dtriangle(0, 0, 2, 0)*dtriangle(1, 0, 2, 0)), nLL_triangle(c(0,1), 0, 2, 0))
  # when a < c = b, then the mle of b is max(x)
  expect_equal(-log(dtriangle(2, 0, 2, 2)), nLL_triangle(2, 0, 2, 2))
  expect_equal(-log(dtriangle(2, 0, 2, 2)*dtriangle(1, 0, 2, 2)), nLL_triangle(c(2,1), 0, 2, 2))
  # when a < c < b, then the mle of a or b sill might be at the min and max
  expect_equal(-log(dtriangle(0, 0, 2, 1)), nLL_triangle(0, 0, 2, 1))
  expect_equal(-log(dtriangle(0, 0, 2, 1)*dtriangle(1, 0, 2, 1)), nLL_triangle(c(0,1), 0, 2, 1))
  expect_equal(-log(dtriangle(2, 0, 2, 1)), nLL_triangle(2, 0, 2, 1))
  expect_equal(-log(dtriangle(2, 0, 2, 1)*dtriangle(1, 0, 2, 1)), nLL_triangle(c(2,1), 0, 2, 1))

  # check that the nLL also integrates to 1 for each type of triangle
  int1 <- integrate(f = function(x) sapply(x, function(xi) exp(-1*nLL_triangle(xi, 0, 2, 1))), lower = 0, upper = 2)
  expect_equal(1, int1$value, tolerance = int1$abs.error)
  int1 <- integrate(f = function(x) sapply(x, function(xi) exp(-1*nLL_triangle(xi, 0, 2, 2))), lower = 0, upper = 2)
  expect_equal(1, int1$value, tolerance = int1$abs.error)
  int1 <- integrate(f = function(x) sapply(x, function(xi) exp(-1*nLL_triangle(xi, 0, 2, 0))), lower = 0, upper = 2)
  expect_equal(1, int1$value, tolerance = int1$abs.error)
})

test_that("gradient_nLL_triangle_given_c works", {
  g <- gradient_nLL_triangle_given_c(xtest_small, 0, 1, 0.3)

  expect_equal(2, length(g))
  expect_true(g[1] < 0 & g[2] > 0)

  expect_equal(c(0, 0.5), gradient_nLL_triangle_given_c(1, 0, 2, 2))
  expect_equal(c(-0.5, 0), gradient_nLL_triangle_given_c(1, 0, 2, 0))
  expect_equal(c(-0.5, 0.5), gradient_nLL_triangle_given_c(1, 0, 2, 1))
})

test_that("hessian_nLL_triangle_given_c works", {
  h <- hessian_nLL_triangle_given_c(xtest_small, 0, 1, 0.3)

  expect_equal(c(2, 2), dim(h))
  expect_true(all(diag(h) > 0))
  expect_equal(h[1,2], h[2,1])

  expect_equal(c(-0.25, 0.75, 0.75, -0.75), c(hessian_nLL_triangle_given_c(c(0,1,2), 0, 2, 2)))
  expect_equal(c(-0.75, 0.75, 0.75, -0.25), c(hessian_nLL_triangle_given_c(c(0,1,2), 0, 2, 0)))
  expect_equal(c(-1.75, 0.75, 0.75, -1.75), c(hessian_nLL_triangle_given_c(c(0,1,2), 0, 2, 1)))

  h <- hessian_nLL_triangle_given_c(xtest_small, 0, 0.81, 0.81)
  expect_equal(c(2, 2), dim(h))
  expect_equal(h[1,2], h[2,1])
})

test_that("triangle_mle_ab_given_c works", {
  temp <- triangle_mle_ab_given_c(xtest_small, 0.3, debug = TRUE)

  expect_true(temp$a < min(xtest_small))
  expect_true(temp$b > max(xtest_small))
  expect_equal(0, temp$optim$convergence)
  expect_equal(c(2, 2), dim(temp$hessian_ab))
  expect_true(all(diag(temp$hessian_ab) > 0))
  expect_equal(temp$hessian_ab[1,2], temp$hessian_ab[2,1])
})

test_that("f_rth_order_stat works", {
  temp <- integrate(f_rth_order_stat, lower = 0, upper = 1, n = 10, r = 3, a = 0, b = 1, c = 0.5)
  expect_equal(1, temp$value)

  expect_equal(rep(0, 4), f_rth_order_stat(c(-1, 0, 1, 2), 10, 3, 0, 1, 0.5))

  expect_true(all(f_rth_order_stat(seq(0.1, 0.9, length=1000), 10, 3, 0, 1, 0.5) > 0))

  expect_equal(rep(0, 4), f_rth_order_stat(c(1, 2, 0, -1), 5, 5, 0, 1, 0.5))
})

test_that("mean_rth_order_stat works", {
  temp <- mean_rth_order_stat(10, 5, 0, 1, 0.5)
  expect_true(temp >= 0 & temp <= 1)
  expect_true(temp < 0.5)

  temp <- mean_rth_order_stat(10, 6, 0, 1, 0.5)
  expect_true(temp > 0.5)

  temp <- mean_rth_order_stat(10, 1, 0, 1, 0.5)
  expect_true(temp >= 0 & temp <= 1)

  temp <- mean_rth_order_stat(10, 10, 0, 1, 0.5)
  expect_true(temp >= 0 & temp <= 1)

  expect_equal(mean_rth_order_stat_numeric(10, 5, 1, 3, 2),
               mean_rth_order_stat(10, 5, 1, 3, 2), tolerance = 1E-9)
  expect_equal(mean_rth_order_stat_numeric(10, 5, 1, 3, 1),
               mean_rth_order_stat(10, 5, 1, 3, 1), tolerance = 1E-9)
  expect_equal(mean_rth_order_stat_numeric(10, 5, 1, 3, 3),
               mean_rth_order_stat(10, 5, 1, 3, 3), tolerance = 1E-9)
  expect_equal(mean_rth_order_stat_numeric(10, 1, 1, 3, 2),
               mean_rth_order_stat(10, 1, 1, 3, 2), tolerance = 1E-9)
  expect_equal(mean_rth_order_stat_numeric(10, 10, 1, 3, 2),
               mean_rth_order_stat(10, 10, 1, 3, 2), tolerance = 1E-9)

  expect_true(mean_rth_order_stat(200, 100, 0, 1, 0.5) < 1)
  expect_true(mean_rth_order_stat(200, 100, 0, 1, 0.5) > 0)
})

test_that("mean_rth_order_stat matches multiple precision", {
  testthat::skip_if_not_installed("Rmpfr")

  expect_equal(mean_rth_order_stat(10, 5, 0, 1, 0.5),
               mean_rth_order_stat_rmpfr(10, 5, 0, 1, 0.5), tolerance = 1E-9)

  # a < c < b
  for (r in 1:10) {
    expect_equal(mean_rth_order_stat(10, r, 1, 3, 2),
                 mean_rth_order_stat_rmpfr(10, r, 1, 3, 2), tolerance = 1E-4)
  }
  # a == c < b
  for (r in 1:10) {
    expect_equal(mean_rth_order_stat(10, r, 1, 3, 1),
                 mean_rth_order_stat_rmpfr(10, r, 1, 3, 1), tolerance = 1E-9)
  }
  # a < c == b
  for (r in 1:10) {
    expect_equal(mean_rth_order_stat(10, r, 1, 3, 3),
                 mean_rth_order_stat_rmpfr(10, r, 1, 3, 3), tolerance = 1E-9)
  }

  expect_equal(mean_rth_order_stat_base(10, 5, 0, 1, 0.5),
               mean_rth_order_stat_rmpfr(10, 5, 0, 1, 0.5), tolerance = 1E-9)

  expect_equal(0.5, mean_rth_order_stat_base(11, 6, 0, 1, 0.5))
  expect_equal(0.5, mean_rth_order_stat_numeric(11, 6, 0, 1, 0.5))
  expect_equal(0.5, mean_rth_order_stat_rmpfr(11, 6, 0, 1, 0.5))
})

test_that("variance_rth_order_stat works", {
  expect_true(variance_rth_order_stat(10, 1, 0, 1, 0.5) >= 0)
  expect_true(variance_rth_order_stat(10, 2, 0, 1, 0.5) >= 0)
  expect_true(variance_rth_order_stat(10, 5, 0, 1, 0.5) >= 0)
  expect_true(variance_rth_order_stat(10, 8, 0, 1, 0.5) >= 0)
  expect_true(variance_rth_order_stat(10, 9, 0, 1, 0.5) >= 0)
  expect_true(variance_rth_order_stat(10, 10, 0, 1, 0.5) >= 0)

  expect_equal(variance_rth_order_stat_numeric(10, 5, 1, 3, 2),
               variance_rth_order_stat(10, 5, 1, 3, 2), tolerance = 1E-9)
  expect_equal(variance_rth_order_stat_numeric(10, 5, 1, 3, 1),
               variance_rth_order_stat(10, 5, 1, 3, 1), tolerance = 1E-9)
  expect_equal(variance_rth_order_stat_numeric(10, 5, 1, 3, 3),
               variance_rth_order_stat(10, 5, 1, 3, 3), tolerance = 1E-9)
  expect_equal(variance_rth_order_stat_numeric(10, 1, 1, 3, 2),
               variance_rth_order_stat(10, 1, 1, 3, 2), tolerance = 1E-9)
  expect_equal(variance_rth_order_stat_numeric(10, 10, 1, 3, 2),
               variance_rth_order_stat(10, 10, 1, 3, 2), tolerance = 1E-9)
})

test_that("variance_rth_order_stat matches multiple precision", {
  testthat::skip_if_not_installed("Rmpfr")

  expect_equal(variance_rth_order_stat(10, 5, 0, 1, 0.5),
               variance_rth_order_stat_rmpfr(10, 5, 0, 1, 0.5), tolerance = 1E-9)

  # a < c < b
  for (r in 1:10) {
    expect_equal(variance_rth_order_stat(10, r, 1, 3, 2),
                 variance_rth_order_stat_rmpfr(10, r, 1, 3, 2), tolerance = 1E-3)
  }
  # a == c < b
  for (r in 1:10) {
    expect_equal(variance_rth_order_stat(10, r, 1, 3, 1),
                 variance_rth_order_stat_rmpfr(10, r, 1, 3, 1), tolerance = 1E-9)
  }
  # a < c == b
  for (r in 1:10) {
    expect_equal(variance_rth_order_stat(10, r, 1, 3, 3),
                 variance_rth_order_stat_rmpfr(10, r, 1, 3, 3), tolerance = 1E-9)
  }
})

test_that("triangle_mle works", {
  temp <- triangle_mle(xtest_small)

  expect_true(temp$coef[1] <= min(xtest_small))
  expect_true(temp$coef[2] >= max(xtest_small))
  # expect_equivalent(0.3, temp$coef[3]) # this isn't true under the left skewwed triangle allowance

  expect_equal(c(3, 3), dim(temp$vcov))
  expect_true(all(diag(temp$vcov) >= 0) | all(is.na(diag(temp$vcov))))  ### wouild like to do better here
  expect_equal(temp$vcov[1,2], temp$vcov[2,1])

  expect_equal(8, temp$nobs)
  expect_true(all(temp$x == xtest_small))

  expect_equal("triangle_mle", class(temp))

  expect_equivalent(nLL_triangle(xtest_small, temp$coef[1], temp$coef[2], temp$coef[3]),
               temp$min)
  expect_equal(0, temp$details$convergence)

  # check that triangles that cross 0 are fine
  set.seed(19393)
  xs <- rtriangle(40, -1, 5, 4)
  expect_no_error(triangle_mle(xs))

  xs <- rtriangle(40, -5, 4,-1)
  expect_no_error(mle2 <- triangle_mle(xs))
  ci <- confint(mle2)
  expect_gte(ci[1,2], -5)
  expect_lte(ci[1,1], -5)
  expect_gte(ci[2,2], 4)
  expect_lte(ci[2,1], 4)
  expect_gte(ci[3,2], -1)
  expect_lte(ci[3,1], -1)

  xs <- rtriangle(40, -5, -1, -3)
  expect_no_error(mle2 <- triangle_mle(xs))
})

test_that("standard_triangle_mle works", {
  temp <- standard_triangle_mle(xtest_small)

  expect_equivalent(0, temp$coef[1])
  expect_equivalent(1, temp$coef[2])
  expect_equivalent(0.3, temp$coef[3])

  expect_equal(c(3, 3), dim(temp$vcov))
  expect_true(all(diag(temp$vcov) >= 0))
  expect_equal(temp$vcov[1,2], temp$vcov[2,1])

  expect_equal(8, temp$nobs)
  expect_true(all(temp$x == xtest_small))

  expect_equal("triangle_mle", class(temp))

  expect_equivalent(nLL_triangle(xtest_small, temp$coef[1], temp$coef[2], temp$coef[3]),
                    temp$min)
})

test_that("breakdown nLL problems", {
  mom1 <- triangle_mom(xtest_small, type = 1)
  triangle:::nLL_triangle(xtest_small, mom1["a"], mom1["b"], mom1["c"]) # inf because a=min(x) and b=max(x)

  # start with mom type 2
  mom2 <- triangle_mom(xtest_small, type = 2)
  triangle:::nLL_triangle(xtest_small, mom2["a"], mom2["b"], mom2["c"])
  # optimize from there
  mle_c0 <- triangle:::triangle_mle_c_given_ab(xtest_small, mom2["a"], mom2["b"])
  mle_ab0 <- triangle:::triangle_mle_ab_given_c(xtest_small, mle_c0$c_hat, start = c(mom2["a"], mom2["b"]))
  mle_c1 <- triangle:::triangle_mle_c_given_ab(xtest_small, mle_ab0$a, mle_ab0$b)
  mle_ab1 <- triangle:::triangle_mle_ab_given_c(xtest_small, mle_c1$c_hat, start = c(mle_ab0$a, mle_ab0$b))
  mle_c2 <- triangle:::triangle_mle_c_given_ab(xtest_small, mle_ab1$a, mle_ab1$b)
  mle_ab2 <- triangle:::triangle_mle_ab_given_c(xtest_small, mle_c2$c_hat, start = c(mle_ab1$a, mle_ab1$b))
  triangle:::nLL_triangle(xtest_small, mle_ab0$a, mle_ab0$b, mle_c0$c_hat)
  triangle:::nLL_triangle(xtest_small, mle_ab1$a, mle_ab1$b, mle_c1$c_hat)
  triangle:::nLL_triangle(xtest_small, mle_ab2$a, mle_ab2$b, mle_c2$c_hat)
  # end up with c at 0.3 and a around 0 and b around 1
  expect_equal(mle_c1$c_hat, mle_c2$c_hat)
  expect_equal(mle_ab1$optim$value, mle_ab2$optim$value)
  # BIC
  3*log(length(xtest_small)) - 2*-1*triangle:::nLL_triangle(xtest_small, mle_ab2$a, mle_ab2$b, mle_c2$c_hat)
  # AIC
  2*3 - 2*-1*triangle:::nLL_triangle(xtest_small, mle_ab2$a, mle_ab2$b, mle_c2$c_hat)
  prod(dtriangle(xtest_small, mle_ab2$a, mle_ab2$b, mle_c2$c_hat))

  # however, the triangle with a = -0.1 and b = c = 0.8 has a smaller nLL and smaller BIC
  mle <- triangle_mle(xtest_small)
  triangle:::nLL_triangle(xtest_small, coef(mle)["a"], coef(mle)["b"], coef(mle)["c"])
  # BIC
  2*log(length(xtest_small)) - 2*-1*triangle:::nLL_triangle(xtest_small, coef(mle)["a"], coef(mle)["b"], coef(mle)["c"])
  # AIC
  2*2 - 2*-1*triangle:::nLL_triangle(xtest_small, coef(mle)["a"], coef(mle)["b"], coef(mle)["c"])
  prod(dtriangle(xtest_small, coef(mle)["a"], coef(mle)["b"], coef(mle)["c"]))

})

# Copyright 2026 Rob Carnell

test_that("summary works", {
  summ <- summary(mle1)

  expect_equal(summ@coef[,1], coef(mle1))
  expect_equal(summ@coef[,2], sqrt(diag(vcov(mle1))))

  expect_equivalent(2 * nLL_triangle(xtest, mle1$coef[1], mle1$coef[2], mle1$coef[3]),
               summ@m2logL)
})

test_that("print works", {
  expect_output(print(mle1))
})

test_that("coef works", {
  cf <- coef(mle1)

  expect_equal(mle1$coef, cf)
})

test_that("logLik works", {
  ll <- logLik(mle1)

  expect_equivalent(-1*nLL_triangle(xtest, mle1$coef[1], mle1$coef[2], mle1$coef[3]),
                    as.numeric(ll))
  expect_equal("logLik", class(ll))
  expect_equal(3, attr(ll, "df"))
})

test_that("AIC works", {
  aic <- AIC(mle1)

  expect_equivalent(2 * nLL_triangle(xtest, mle1$coef[1], mle1$coef[2], mle1$coef[3]) + 2 * 3,
                    aic)
})

test_that("BIC works", {
  bic <- BIC(mle1)

  expect_equivalent(2 * nLL_triangle(xtest, mle1$coef[1], mle1$coef[2], mle1$coef[3]) + log(length(xtest)) * 3,
                    bic)
})

test_that("vcov works", {
  v <- vcov(mle1)

  expect_equivalent(mle1$vcov, v)
})

test_that("profile works", {
  prof <- profile(mle1)

  expect_equivalent("profile.mle", class(prof))
  expect_equal(3, length(prof@profile))
  expect_equal(3, ncol(prof@profile$a$par.vals))

  mle2 <- standard_triangle_mle(xtest)
  prof <- profile(mle2)

  expect_equivalent("profile.mle", class(prof))
  expect_equal(3, length(prof@profile))
  expect_equal(3, ncol(prof@profile$a$par.vals))
})

test_that("confint works", {
  cfi <- confint(mle1, level = 0.95)

  expect_equal(c(3,2), dim(cfi))
  expect_true(cfi[1,1] < 0 & 0 < cfi[1,2])
  expect_true(cfi[2,1] < 1 & 1 < cfi[2,2])

  cfi <- confint(mle1, level = 0.9)

  expect_true(cfi[1,1] < 0 & 0 < cfi[1,2])
  expect_true(cfi[2,1] < 1 & 1 < cfi[2,2])
  expect_true(cfi[3,1] < 0.5 & 0.5 < cfi[3,2])
})

# Extracted from test-mle.R:36

# setup ------------------------------------------------------------------------
library(testthat)
test_env <- simulate_test_env(package = "triangle", path = "..")
attach(test_env, warn.conflicts = FALSE)

# test -------------------------------------------------------------------------
expect_equal(-sum(log(dtriangle(xtest_small, 0, 1, 0.3))),
               nLL_triangle(xtest_small, 0, 1, 0.3),
               tolerance = 1E-5)
expect_equal(-log(1/2), nLL_triangle(1, 0, 2, 2, debug = TRUE))
expect_equal(-log(1/2), nLL_triangle(1, 0, 2, 0))
expect_equal(-log(1), nLL_triangle(1, 0, 2, 1))
expect_equal(-log(dtriangle(0, 0, 2, 0)), nLL_triangle(0, 0, 2, 0))
expect_equal(-log(dtriangle(0, 0, 2, 0)*dtriangle(1, 0, 2, 0)), nLL_triangle(c(0,1), 0, 2, 0))
expect_equal(-log(dtriangle(2, 0, 2, 2)), nLL_triangle(2, 0, 2, 2))

# Copyright 2018 Rob Carnell

#' @include dtriangle.R
#' @rdname triangle
#' @export
qtriangle <- function(p, a=0, b=1, c=(a + b)/2) {
  p1 <- length(p)
  a1 <- length(a)
  b1 <- length(b)
  c1 <- length(c)

  qTest <- function(X){
    # X = c(p, a, b, c)
    if (any(is.na(X))) { # is.na is TRUE for NA, NaN, and FALSE
      if (any(is.nan(X))) return(NaN) # to conform to qunif
      else return(NA) # to conform to qunif
    } else if (X[2] > X[4] | X[3] < X[4]) {
      warning("values required to be  a <= c <= b (at least one strict inequality)")
      return(NaN) # to conform to behavior of qunif
    } else if (X[1] < 0 | X[1] > 1) {
      warning("at least one p is outside [0,1]")
      return(NaN) # to conform to behavior of qunif
    } else if (any(is.infinite(X[2:4]))) {
      warning("at least one parameters is Infinite")
      return(NaN)
    } else if ((X[2] != X[4] &&
               (X[2] + sqrt(X[1]*(X[3] - X[2])*(X[4] - X[2]))) <= X[4]) |
              (X[2] == X[4] &&
               (X[2] + sqrt(X[1]*(X[3] - X[2])*(X[4] - X[2]))) < X[4])) {
      return(X[2] + sqrt(X[1]*(X[3] - X[2])*(X[4] - X[2])))
    } else if ((X[2] != X[4] &&
               (X[3] - sqrt((1 - X[1])*(X[3] - X[2])*(X[3] - X[4]))) > X[4]) |
              (X[2] == X[4] &&
               (X[3] - sqrt((1 - X[1])*(X[3] - X[2])*(X[3] - X[4]))) >= X[4])) {
      return(X[3] - sqrt((1 - X[1])*(X[3] - X[2])*(X[3] - X[4])))
    } else if (X[2] != X[4] && (X[2] + sqrt(X[1]*(X[3] - X[2])*(X[4] - X[2]))) - X[4] <= (X[3] - X[2])*.Machine$double.eps)
    {
      return(X[4])
    } else stop("Unexpected Result")
  }

  k <- max(p1, a1, b1, c1)
  if (k == 1) return(qTest(c(p,a,b,c)))

  params <- matrix(nrow = k, ncol = 4)
  tryCatch(
  {
    params[,1] <- p
    params[,2] <- a
    params[,3] <- b
    params[,4] <- c
  }, error = function(X) {
    stop(paste(" -- Argument Lengths: length of p = ", p1,
                ", a = ", a1, ", b = ", b1, ", c = ", c1, " -- ", X, sep = ""))
  })

  return(apply(params, 1, qTest))
}

# Copyright 2024 Robert Carnell

#' Triangle distribution method of moments estimate
#'
#' @param x triangle distribution sample
#' @param na.rm whether to remove NA samples.  Default is FALSE.
#' @param type the type of method of moments.  Type 1 uses the min and max.  Type2
#' minimizes the distance from the calculated mean, variance, and skewness to the
#' sample mean, variance, and skewness
#'
#' @return a named vector of the parameter estimates
#' @export
#'
#' @examples
#' set.seed(1204)
#' x <- rtriangle(20, 0, 2, 1.5)
#' triangle_mom(x)
triangle_mom <- function(x, na.rm = FALSE, type = 1)
{
  ap <- min(x, na.rm = na.rm)
  bp <- max(x, na.rm = na.rm)
  mean_x <- mean(x, na.rm = na.rm)
  if (type == 1)
  {
    cp <- 3*mean_x - ap - bp
    return(c(a=ap, b=bp, c=cp))
  } else if (type == 2)
  {
    var_x <- var(x, na.rm = na.rm)
    skew_x <- .skew(x, na.rm = na.rm)
    opt_fn <- function(theta) {
      a <- theta[1]
      b <- theta[2]
      c <- theta[3]
      if (a >= b | c < a | c > b) return(.Machine$double.xmax)
      pred_mean <- (a + b + c) / 3
      pred_var <- (a^2 + b^2 + c^2 - a*b - a*c - b*c) / 18
      pred_skew <- sqrt(2) * (a + b - 2*c) * (2*a - b - c) * (a - 2*b + c) / 5 / (18*pred_var)^(3/2)
      return((pred_mean - mean_x)^2 + (pred_var - var_x)^2 + (pred_skew - skew_x)^2)
    }
    o <- optim(par = c(ap, bp, 3*mean_x - ap - bp), fn = opt_fn)
    if (o$convergence != 0) {
      warning("method of moments did not converge.  check attr(*, 'Optim')")
    }
    ret_value <- c(a=o$par[1], b=o$par[2], c=o$par[3])
    attr(ret_value, "optim") <- o
    return(ret_value)
  } else {
    stop("type not recognized")
  }
}

.skew <- function (x, na.rm = FALSE)
{
  if (any(is.na(x))) {
    if (na.rm) {
      x <- x[which(!is.na(x))]
    }
    else return(NA)
  }
  n <- length(x)
  x <- x - mean(x)
  return(sqrt(n) * sum(x^3)/(sum(x^2)^(3/2)))
}

# Copyright 2026 Rob Carnell

test_that("method of moments works", {
  set.seed(103930)
  x <- rtriangle(20, 0, 2, 1.5)
  mom <- triangle_mom(x)

  expect_equal(3, length(mom))
  expect_true(mom[1] <= mom[3])
  expect_true(mom[2] >= mom[3])
  expect_true(mom[1] <= mom[2])

  mom2 <- triangle_mom(x, type = 1)

  expect_equal(mom, mom2)

  mom3 <- triangle_mom(x, type = 2)
  expect_equal(3, length(mom))
  expect_true(mom3[1] <= mom3[3])
  expect_true(mom3[2] >= mom3[3])
  expect_true(mom3[1] <= mom3[2])
  expect_true(abs(mean(x) -(mom3["a"] + mom3["b"] + mom3["c"])/3) < 0.1)

  x <- rtriangle(20, 1, 3, 1)
  mom3 <- triangle_mom(x, type = 2)
  expect_equal(3, length(mom))
  expect_true(mom3[1] <= mom3[3])
  expect_true(mom3[2] >= mom3[3])
  expect_true(mom3[1] <= mom3[2])
  expect_true(abs(mean(x) -(mom3["a"] + mom3["b"] + mom3["c"])/3) < 0.1)

  x <- rtriangle(20, 1, 3, 3)
  mom3 <- triangle_mom(x, type = 2)
  expect_equal(3, length(mom))
  expect_true(mom3[1] <= mom3[3])
  expect_true(mom3[2] >= mom3[3])
  expect_true(mom3[1] <= mom3[2])
  expect_true(abs(mean(x) -(mom3["a"] + mom3["b"] + mom3["c"])/3) < 0.1)
})

test_that("repeated mom", {
  skip_on_cran()
  skip_on_ci()

  set.seed(1393)
  b <- replicate(200, {
    x <- rtriangle(100, 0, 2, 1.5)
    triangle_mom(x)
  })
  expect_true(all(abs(apply(b, 1, mean) - c(a=0, b=2, c=1.5)) < 0.2))

  set.seed(1393)
  b <- replicate(200, {
    x <- rtriangle(100, 0, 2, 1.5)
    triangle_mom(x, type = 2)
  })
  expect_true(all(abs(apply(b, 1, mean) - c(a=0, b=2, c=1.5)) < 0.2))

  set.seed(1393)
  b <- replicate(200, {
    x <- rtriangle(100, 0, 2, 2)
    triangle_mom(x)
  })
  expect_true(all(abs(apply(b, 1, mean) - c(a=0, b=2, c=2)) < 0.2))

  set.seed(1393)
  b <- replicate(200, {
    x <- rtriangle(100, 0, 2, 2)
    triangle_mom(x, type = 2)
  })
  expect_true(all(abs(apply(b, 1, mean) - c(a=0, b=2, c=2)) < 0.2))

})

# Copyright 2024 Robert Carnell

#' Utility Methods for S3 class triangle_mle
#'
#' @param object class triangle_mle from a call to \code{triangle_mle()}
#' @param x the \code{triangle_mle} object
#' @param ... not used except for \code{print} (other arguments passed to \code{printCoefmat})
#'
#' @rdname mle-utils
#'
#' @return an object of class summary.mle
#' @method summary triangle_mle
#' @export
#'
#' @importClassesFrom stats4 summary.mle
#' @importFrom stats4 show
#' @importFrom methods new
#'
#' @examples
#' set.seed(1234)
#' x <- rtriangle(100, 0, 1, 0.5)
#' mle1 <- triangle_mle(x)
#' summary(mle1)
#' print(mle1)
#' coef(mle1)
#' logLik(mle1)
#' AIC(mle1)
#' BIC(mle1)
#' vcov(mle1)
#' \dontrun{
#'   prof <- profile(mle1)
#'   stats4::plot(prof)
#'   confint(mle1, 1:3, level = 0.95)
#' }
summary.triangle_mle <- function(object, ...)
{
  vars <- diag(object$vcov)
  # it is possible to have negative numbers in the vcov because of numerical precision
  vars[which(vars < 0)] <- NA
  cmat <- cbind(Estimate = object$coef, `Std. Error` = sqrt(vars))
  m2logL <- 2 * object$min
  methods::new("summary.mle", call = object$call, coef = cmat, m2logL = m2logL)
}

#' @rdname mle-utils
#' @return \code{print.triangle_mle}: \code{x} invisibly
#' @export
#'
#' @importFrom stats printCoefmat
#' @importFrom stats4 coef vcov
print.triangle_mle <- function(x, ...)
{
  vars <- diag(stats4::vcov(x))
  # it is possible to have negative numbers in the vcov because of numerical precision
  vars[which(vars < 0)] <- NA
  cat("Triangle Maximum Likelihood Estimates")
  cat("\n\nCall: ", deparse(x$call), "\n")
  cat("\nEstimates:\n")
  cmat <- cbind(stats4::coef(x), sqrt(vars))
  colnames(cmat) <- c("Estimate", "Std.Err")
  stats::printCoefmat(cmat, ...)
  cat("\nConvergence Code: ", ifelse(all(is.na(x$details)), NA, x$details$convergence))
  cat("\n\t", ifelse(all(is.na(x$details)), "", x$details$message))
  invisible(x)
}

#' @rdname mle-utils
#' @return \code{coef.triangle_mle}: a vector of coefficients
#' @method coef triangle_mle
#' @export
coef.triangle_mle <- function(object, ...) object$coef

#' @rdname mle-utils
#' @return \code{logLik.triangle_mle}: an object of class \code{logLik}
#' @export
logLik.triangle_mle <- function(object, ...)
{
  structure(-1*object$min,
            df = length(object$coef),
            class = "logLik")
}

#' @rdname mle-utils
#' @param k the penalty per parameter to be used; the default \code{k = 2}
#' @return \code{AIC.triangle_mle}: the AIC
#' @export
AIC.triangle_mle <- function(object, ..., k=2)
{
  lls <- logLik.triangle_mle(object)
  -2 * as.numeric(lls) + k * attr(lls, "df")
}

#' @rdname mle-utils
#' @return \code{BIC.triangle_mle}: the BIC
#' @importFrom stats BIC
#' @export
BIC.triangle_mle <- function(object, ...)
{
  lls <- logLik.triangle_mle(object)
  nobs <- object$nobs
  -2 * as.numeric(lls) + log(nobs) * attr(lls, "df")
}

#' @rdname mle-utils
#' @return \code{vcov.triangle_mle}: the variance co-variance matrix
#' @method vcov triangle_mle
#' @export
vcov.triangle_mle <- function(object, ...)
{
  object$vcov
}

#' @rdname mle-utils
#' @return \code{profile.triangle_mle}: an object of class \code{profile.mle}
#' @param fitted an object of class triangle_mle
#' @export
#'
#' @importClassesFrom stats4 profile.mle
#' @importFrom methods new
#' @importFrom stats qchisq
profile.triangle_mle <- function(fitted, ...)
{
  #xtest <- c(0.1, 0.25, 0.3, 0.4, 0.45, 0.6, 0.75, 0.8)
  #fitted <- triangle_mle(xtest)
  #fitted <- standard_triangle_mle(xtest, debug = TRUE)
  alpha = 0.01
  zmax = sqrt(stats::qchisq(1-alpha, 1))

  p <- length(fitted$coef)
  stopifnot(p == 3)
  summ <- summary(fitted)
  std.err <- summ@coef[, "Std. Error"]
  B0 <- fitted$coef
  Pnames <- names(B0)

  maxa <- min(fitted$x) - .Machine$double.eps * min(fitted$x)
  minb <- max(fitted$x) + .Machine$double.eps * max(fitted$x)

  sox <- sort(fitted$x)
  r <- which(sox == B0[3])

  prof <- vector("list", length = 3)
  names(prof) <- Pnames

  calc_z <- function(a, b, c, mysign)
  {
    nLLi <- nLL_triangle(fitted$x, a, b, c)
    nLLbase <- nLL_triangle(fitted$x, B0[1], B0[2], B0[3])
    if (nLLi < nLLbase) return(0)
    ret <- mysign * sqrt(2 * (nLLi - nLLbase))
    return(ret)
  }

  # up
  abc <- list(a = B0[1], b = B0[2], c = B0[3])
  z <- list(a = 0, b = 0, c = 0)
  for (j in 1:3)
  {
    # if any parameters have been fixed and given 0 variance
    #   set up a dummy return value with at least 4 points.
    if (std.err[j] == 0)
    {
      z[[j]] <- c(z[[j]], 1000, -1000, 2000)
      # original estimates, a little larger, a little lower, larger again
      abc[[j]] <- c(abc[[j]],
                    ifelse(B0[j] == 0, B0[j] + 0.000001, B0[j] + B0[j]*0.000001),
                    ifelse(B0[j] == 0, B0[j] - 0.000001, B0[j] - B0[j]*0.000001),
                    ifelse(B0[j] == 0, B0[j] + 0.000002, B0[j] + B0[j]*0.000002))
      next
    }
    ri <- r
    zi <- 0
    abci <- B0[j]
    while (zi < zmax)
    {
      ri <- ri + 1
      if (j == 3 & ri > length(fitted$x))
        break
      abci <- ifelse(j == 1, (maxa + abci) / 2, ifelse(j == 2, abci + std.err[j], sox[ri]))
      zi <- ifelse(j == 1, calc_z(abci, B0[2], B0[3], 1),
                   ifelse(j == 2, calc_z(B0[1], abci, B0[3], 1),
                          calc_z(B0[1], B0[2], abci, 1)))
      z[[j]] <- c(z[[j]], zi)
      abc[[j]] <- c(abc[[j]], abci)
    }
    # down
    ri <- r
    zi <- 0
    abci <- B0[j]
    while (zi > -zmax)
    {
      ri <- ri - 1
      if (j == 3 & ri < 1)
        break
      abci <- ifelse(j == 1, abci - std.err[j], ifelse(j == 2, (minb + abci) / 2, sox[ri]))
      zi <- ifelse(j == 1, calc_z(abci, B0[2], B0[3], -1),
                   ifelse(j == 2, calc_z(B0[1], abci, B0[3], -1),
                          calc_z(B0[1], B0[2], abci, -1)))
      z[[j]] <- c(z[[j]], zi)
      abc[[j]] <- c(abc[[j]], abci)
    }
  }

  for (j in 1:3)
  {
    # eliminate duplicate z
    ind <- which(duplicated(z[[j]]))
    if (length(ind) > 0)
    {
      z[[j]] <- z[[j]][-ind]
      abc[[j]] <- abc[[j]][-ind]
    }

    par.vals.i <- matrix(rep(B0, each = length(z[[j]])), nrow = length(z[[j]]), ncol = 3, dimnames = list(NULL, Pnames))
    par.vals.i[,j] <- abc[[j]]

    ord <- order(z[[j]])
    prof[[j]] <- data.frame(z = z[[j]][ord])
    prof[[j]]$par.vals <- par.vals.i[ord, , drop = FALSE]
  }
  methods::new("profile.mle", profile = prof, summary = summ)
}

# Note that extra documentation about this being an S3 method (@exportS3Method and @method) was required to product the correct .Rd file
# @exportS3Method stats::confint

#' @rdname mle-utils
#' @param parm parameters to be given confidence intervals passed to \code{stats4::confint}
#' @param level confidence interval level passed to \code{stats4::confint}
#' @return \code{confint.triangle_mle}: a matrix of parameter confidence intervals
#' @method confint triangle_mle
#' @export
#' @importClassesFrom stats4 profile.mle
#' @importFrom stats4 confint
confint.triangle_mle <- function(object, parm, level = 0.95, ...)
{
  prof <- profile.triangle_mle(object)
  stats4::confint(prof, parm, level, ...)
}

# Copyright Robert Carnell 2022

#' Quantile-Quantile Plot for Triangle Distributed Data
#'
#' @param y the triangle distributed sample
#' @param a the theoretical distribution triangle minimum parameter
#' @param b the theoretical distribution triangle maximum parameter
#' @param c the theoretical distribution triangle mode parameter
#' @param main the plot title
#' @param xlab the x-axis label
#' @param ylab the y-axis label
#' @param ... other parameters passed to \code{qqplot}
#'
#' @return a list of x-y coordinates on the plot
#' @export
#'
#' @importFrom stats qqplot ppoints
#' @importFrom graphics abline
#'
#' @examples
#' set.seed(10304)
#' xtest <- rtriangle(100, 1, 5, 2)
#' theta <- coef(triangle_mle(xtest))
#' qqtriangle(xtest, theta[1], theta[2], theta[3])
qqtriangle <- function(y, a, b, c, main = "Triangle Q-Q Plot",
                       xlab = "Theoretical Quantiles",
                       ylab = "Sample Quantiles",
                       ...)
{
  # y <- rtriangle(100, 0, 5, 2)
  leny <- length(y)
  h <- stats::qqplot(qtriangle(stats::ppoints(leny), a, b, c), y, main = main,
                     xlab = xlab, ylab = ylab, ...)
  graphics::abline(0, 1)
  return(invisible(h))
}


#' Compare multiple triangle distributions fits
#'
#' @param y the triangle distributed sample
#' @param cols the colors of the CDF-based estimates, the maximum likelihood estimates, and the method of moments estimates
#' @param main the plot title
#' @param ... other parameters passed to \code{plot.ecdf}
#'
#' @importFrom stats coef ecdf plot.ecdf
#' @importFrom stats4 coef
#' @importFrom graphics lines legend
#'
#' @export
#'
#' @examples
#' set.seed(10304)
#' xtest <- rtriangle(100, 1, 5, 2)
#' compare_triangle_fit(xtest)
compare_triangle_fit <- function(y, cols = c("red", "blue", "green"), main = "Triangle Fit Comparison", ...)
{
  # y <- rtriangle(1000, 0, 1, .3)
  theta1 <- stats::coef(triangle_cdfe(y))
  theta2 <- stats4::coef(triangle_mle(y))
  theta3 <- triangle_mom(y)

  n <- 1000
  basex <- seq(min(y), max(y), length = n)
  stats::plot.ecdf(stats::ecdf(y), main = main, ...)
  graphics::lines(basex, ptriangle(basex, theta1[1], theta1[2], theta1[3]), col = cols[1])
  graphics::lines(basex, ptriangle(basex, theta2[1], theta2[2], theta2[3]), col = cols[2])
  graphics::lines(basex, ptriangle(basex, theta3[1], theta3[2], theta3[3]), col = cols[3])
  graphics::legend("topleft", legend = c("CDF estimates", "MLE", "MOM estimates", "Empirical CDF"),
                   col = c(cols, "black"), lty = 1, pch = c(NA, NA, NA, 19))
}

# Copyright 2026 Rob Carnell

test_that("qqtriangle works", {
  set.seed(10304)
  xtest <- rtriangle(100, 1, 5, 2)
  theta <- coef(triangle_mle(xtest))
  expect_no_error(qqtriangle(xtest, theta[1], theta[2], theta[3]))
})

test_that("compare_triangle_fit works", {
  set.seed(10304)
  xtest <- rtriangle(100, 1, 5, 2)
  expect_no_error(compare_triangle_fit(xtest))
})

# Copyright 2018 Rob Carnell

#' @include dtriangle.R
#' @rdname triangle
#' @importFrom stats runif
#' @export
rtriangle <- function(n=1, a=0, b=1, c=(a + b)/2)
{
  if (length(n) > 1)
  {
    n <- length(n)
  }
  # this is not how runif works runif(4, c(1,2,3), c(1,2,3)) gives 1,2,3,1
  if (length(a) > 1 | length(b) > 1 | length(c) > 1)
  {
    stop("a, b, and c must be atomic")
  }
  if (n < 1 | is.na(n))
  {
    stop(paste("invalid argument: n =", n))
  }
  n <- floor(n)
  if (any(is.na(c(a,b,c))) ||
      any(is.infinite(c(a,b,c))) ||
      any(a > c) ||
      any(b < c) ||
      any(b < a))
  {
    warning("NAs produced")
    return(rep(NaN, times = n)) # to match behavior of runif
  }
  if (a > c | b < c)
  {
    return(rep(NaN, times = n)) # to match behavior of runif
  }

  p <- runif(n)

  if (a != c) {
    # if a = c then i is always true
    i <- which((a + sqrt(p * (b - a)*(c - a))) <= c)
    j <- which((b - sqrt((1 - p) * (b - a) * (b - c))) > c)
  } else {
    i <- which((a + sqrt(p * (b - a)*(c - a))) < c)
    j <- which((b - sqrt((1 - p) * (b - a) * (b - c))) >= c)
  }
  if (length(i) != 0)
    p[i] <- a + sqrt(p[i] * (b - a) * (c - a))
  if (length(j) != 0)
    p[j] <- b - sqrt((1 - p[j]) * (b - a) * (b - c))

  return(p)
}

# Copyright 2026 Rob Carnell

test_that("rtriangle works", {
  expect_true(max(rtriangle(100)) <= 1)
  expect_true(min(rtriangle(100)) >= 0)

  # From a Bug Report, Michael.Scroggie@dse.vic.gov.au, Thursday 10/19/06
  expect_true(!all(0 == rtriangle(1, 0, 1, 0)))
  expect_true(!all(1 == rtriangle(1, 0, 1, 1)))
  expect_true(!all(5 == rtriangle(1, 2, 5, 5)))
  expect_true(!identical(NaN, rtriangle(1, 2, 5, 2)))

  test <- rtriangle(1, 2, 5, 2 + 10^-6)
  expect_true(test > 2 & test < 5)
  test <- rtriangle(1, 2, 5, 2 + 10^-9)
  expect_true(test > 2 & test < 5)
  test <- rtriangle(1, 2, 5, 2 + 10^-12)
  expect_true(test > 2 & test < 5)
  expect_true(!is.nan(rtriangle(1, 2, 5, 2 + 10^-18)))

  expect_true(!all(0 == rtriangle(100, 0, 100, 0)))
  expect_true(!any(is.nan(rtriangle(100, 10, 100, 10))))
  expect_true(!all(-1 == rtriangle(10, -1, 0, -1)))
  expect_true(!all(-2 == rtriangle(10, -5, -2, -2)))
  expect_true(all(1 == rtriangle(10, 1, 1, 1)))
  expect_true(all(-1 == rtriangle(10, -1, -1, -1)))
})

test_that("rtriangle is consistent with runif", {
  # consistent with runif(20.3)
  expect_equal(length(rtriangle(20.3)), 20)
  # runif(c(1,2,3,6)) returns a vector of length 4
  expect_equal(length(rtriangle(c(1,2,3,4), 4, 5, 5)), 4)
  expect_equal(2, length(rtriangle(c(4,5), 0, 2, 1)))

  # runif(-1) errors
  expect_error(rtriangle(-5))
  # runif(2, NaN, 5) returns c(NaN, NaN) and warning
  expect_warning(expect_true(all(is.nan(rtriangle(2, NaN, 3, 1)))))
  expect_warning(expect_length(rtriangle(2, NaN, 3, 1), 2))
  # runif(1, 3, NA) returns NaN and warning
  expect_warning(expect_true(is.nan(rtriangle(1, 3, NA, 4))))
  # runif(NA) errors
  expect_error(rtriangle(NA))
  # runif(Inf) errors
  expect_error(rtriangle(Inf))

  # runif(1, 10, 3) produces NaN with warning
  # a > c
  expect_warning(expect_true(all(is.nan(rtriangle(10, 5, 6, 4)))))
  # b < c
  expect_warning(expect_true(all(is.nan(rtriangle(10, 5, 6, 7)))))
  # runif(1, 1, Inf) returns NaN with a warning
  expect_warning(expect_true(all(is.nan(rtriangle(10, 5, Inf, 7)))))
})

test_that("rtriangle is not consistent with runif", {
  # this is not how runif works, but runif would be difficult to implment
  # runif(4, c(1,2,3), c(1,2,3)) == c(1,2,3,1)
  expect_error(rtriangle(10, c(1,2), 4, 6))
})

# Copyright 2026 Rob Carnell

test_that("triangle_cdfe works", {
  set.seed(10304)
  xtest2 <- rtriangle(100, 1, 5, 2)
  cdfe <- triangle_cdfe(xtest2)

  expect_equal("nls", class(cdfe))
  expect_output(print(cdfe))
  expect_output(print(summary(cdfe)))
  expect_equal(3, length(coef(cdfe)))
  expect_true(coef(cdfe)[1] < coef(cdfe)[2])
  expect_true(coef(cdfe)[1] <= min(xtest2))
  expect_true(coef(cdfe)[2] >= max(xtest2))
  expect_true(coef(cdfe)[3] <= coef(cdfe)[2])
  expect_true(coef(cdfe)[3] >= coef(cdfe)[1])
  expect_true(coef(cdfe)[3] >= min(xtest2))
  expect_true(coef(cdfe)[3] <= max(xtest2))

  # do this so MASS:::confint.nls can be used
  if (requireNamespace("MASS", quietly = TRUE)) {
    expect_equal(c(3, 2), dim(confint(cdfe)))
  }
})

# Copyright 2018 Rob Carnell

#' The Triangle Distribution
#'
#' @description These functions provide information about the triangle
#' distribution on the interval from \code{a} to \code{b} with a maximum at
#' \code{c}.  \code{dtriangle} gives the density, \code{ptriangle} gives the
#' distribution function, \code{qtriangle} gives the quantile function, and
#' \code{rtriangle} generates \code{n} random deviates.
#'
#' @details   All probabilities are lower tailed probabilities.
#' \code{a}, \code{b}, and \code{c} may be appropriate length vectors except in
#' the case of \code{rtriangle}.  \code{rtriangle} is derived from a draw from
#' \code{runif}. The triangle distribution has density:
#' \deqn{f(x) = \frac{2(x-a)}{(b-a)(c-a)}}{f(x) = 2(x-a) / [(b-a)(c-a)]}
#' for \eqn{a \le x < c}{a <= x < c}.
#' \deqn{f(x) = \frac{2(b-x)}{(b-a)(b-c)}}{f(x) = 2(b-x) / [(b-a)(b-c)]}
#' for \eqn{c \le x \le b}{c <= x <= b}.
#' \eqn{f(x) = 0} elsewhere.
#' The mean and variance are:
#' \deqn{E(x) = \frac{(a + b + c)}{3}}{E(x) = (a + b + c) / 3}
#' \deqn{V(x) = \frac{1}{18}(a^2 + b^2 + c^2 - ab - ac - bc)}{V(x) = (a^2 + b^2 + c^2 - ab - ac - bc) / 18}
#'
#' @param x,q vector of quantiles.
#' @param a lower limit of the distribution.
#' @param b upper limit of the distribution.
#' @param c mode of the distribution.
#' @param p vector of probabilities.
#' @param n number of observations. If \code{length(n) > 1}, the length is taken to be the number required.
#'
#' @return \code{dtriangle} gives the density, \code{ptriangle} gives the
#' distribution function, \code{qtriangle} gives the quantile function, and
#' \code{rtriangle} generates random deviates.  Invalid arguments will result
#' in return value \code{NaN} or \code{NA}.
#' @references
#' Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988) \emph{The New S Language}.  Wadsworth & Brooks/Cole.
#' @seealso
#' \code{\link{.Random.seed}} about random number generation,
#' \code{\link{runif}}, etc for other distributions.
#' @keywords distribution
#'
#' @name triangle
#' @export
#'
#' @examples
#' ## view the distribution
#' tri <- rtriangle(100000, 1, 5, 3)
#' hist(tri, breaks=100, main="Triangle Distribution", xlab="x")
#' mean(tri) # 1/3*(1 + 5 + 3) = 3
#' var(tri)  # 1/18*(1^2 + 3^2 + 5^2 - 1*5 - 1*3 - 5*3) = 0.666667
#' dtriangle(0.5, 0, 1, 0.5) # 2/(b-a) = 2
#' qtriangle(ptriangle(0.7)) # 0.7
dtriangle <- function(x, a=0, b=1, c=(a + b)/2)
{
  x1 <- length(x)
  a1 <- length(a)
  b1 <- length(b)
  c1 <- length(c)

  dTest <- function(X)
  {
    xx <- 1
    aa <- 2
    bb <- 3
    cc <- 4
    if (any(is.na(X))) # is.na is TRUE for NA, NaN, and FALSE
    {
      if (any(is.nan(X))) return(NaN) # to conform to dunif
      else return(NA) # to conform to qunif
    } else if (X[aa] > X[cc] | X[bb] < X[cc] | (X[aa]==X[cc] & X[bb]==X[cc]))
    {
      warning("values required to be  a <= c <= b (at least one strict inequality)")
      return(NaN) # to conform to behavior of dunif
    } else if (any(is.infinite(X[aa:cc])))
    {
      # If the statistics are in the right order, then return 0 to be consistent with dunif
      return(0)
    } else if (X[xx] < X[aa]) {
      return(0)
    } else if (X[xx] == X[aa] & X[aa] == X[cc])
    {
      return(2*(X[bb] - X[xx]) / (X[bb] - X[aa]) / (X[bb] - X[cc]))
    } else if (X[aa] != X[cc] & X[xx] < X[cc])
    {
      return(2*(X[xx] - X[aa]) / (X[bb] - X[aa]) / (X[cc] - X[aa]))
    } else if (X[cc] != X[bb] & X[xx] >= X[cc] & X[xx] <= X[bb])
    {
      return(2*(X[bb] - X[xx]) / (X[bb] - X[aa]) / (X[bb] - X[cc]))
    } else if (X[xx] == X[bb] & X[bb] == X[cc])
    {
      return(2*(X[xx] - X[aa]) / (X[bb] - X[aa]) / (X[cc] - X[aa]))
    } else if (X[xx] > X[bb])
    {
      return(0)
    }
  }

  k <- max(x1, a1, b1, c1)
  if (k==1) return(dTest(c(x, a, b, c)))

  params <- matrix(nrow=k, ncol=4)
  tryCatch(
  {
    params[,1] <- x
    params[,2] <- a
    params[,3] <- b
    params[,4] <- c
  }, error = function(X) {
    stop(paste(" -- Argument Lengths: length of x = ", x1,
                ", a = ", a1, ", b = ", b1, ", c = ", c1, " -- ", X, sep=""))
  })

  return(apply(params, 1, dTest))
}

# Copyright 2026 Rob Carnell

test_that("dtriangle", {
  expect_equal(dtriangle(1, 1, 3, 2), 0) # x < a
  expect_equal(dtriangle(3, 0, 3, 2), 0) # x > b
  expect_equal(dtriangle(-1, -1, 3, 2), 0) # x = a
  expect_equal(dtriangle(.5, 0, 1, .5), 2) # = 2 (x - a) / (b-a) / (c-a)

  expect_warning(expect_equal(dtriangle(0, 0, 0, 0), NaN))
  expect_warning(expect_equal(dtriangle(.5, 0, 1, 2), NaN))
  expect_warning(expect_equal(dtriangle(.5, 1, 2, .5), NaN))
  expect_equal(dtriangle(c(1, 3, -1, .5), c(1, 0, -1, 0), c(3, 3, 3, 1),
                               c(2, 2, 2, .5)), c(0, 0, 0, 2))
  expect_warning(expect_equal(dtriangle(c(.5, .6), c(0, 0), c(1, 1),
                                         c(.5, 1.5)), c(2, NaN)))
  # regression test
  expect_true(all(!is.nan(dtriangle(0:10, a = 2, b = 5, c = 2))))
  expect_warning(expect_true(is.nan(dtriangle(.5, 1, 1, 1))))

  # more case testing
  expect_equal(dtriangle(0, 1, 3, 2), 0)
  expect_equal(dtriangle(1, 1, 3, 2), 0)
  expect_equal(dtriangle(1.5, 1, 3, 2), 0.5)
  expect_equal(dtriangle(2, 1, 3, 2), 1)
  expect_equal(dtriangle(2.5, 1, 3, 2), 0.5)
  expect_equal(dtriangle(3, 1, 3, 2), 0)
  expect_equal(dtriangle(4, 1, 3, 2), 0)

  expect_equal(dtriangle(0, 1, 3, 1), 0)
  expect_equal(dtriangle(1, 1, 3, 1), 1)
  expect_equal(dtriangle(2, 1, 3, 1), 0.5)
  expect_equal(dtriangle(3, 1, 3, 1), 0)
  expect_equal(dtriangle(4, 1, 3, 1), 0)

  expect_equal(dtriangle(0, 1, 3, 3), 0)
  expect_equal(dtriangle(1, 1, 3, 3), 0)
  expect_equal(dtriangle(2, 1, 3, 3), 0.5)
  expect_equal(dtriangle(3, 1, 3, 3), 1)
  expect_equal(dtriangle(4, 1, 3, 3), 0)
})

test_that("dtriangle is consistent with dunif", {
  # NaN
  expect_true(is.nan(dunif(0.5, NaN, 1)))
  expect_true(is.nan(dunif(0.5, 0, NaN)))
  expect_true(is.nan(dunif(NaN, 0, 1)))

  expect_true(is.nan(dtriangle(0.5, NaN, 1, 0.5)))
  expect_true(is.nan(dtriangle(0.5, 0, NaN, 0.5)))
  expect_true(is.nan(dtriangle(0.5, 0, 1, NaN)))
  expect_true(is.nan(dtriangle(NaN, 0, 1, 0.5)))

  # NA
  expect_true(is.na(dunif(0.5, NA, 1)))
  expect_true(is.na(dunif(0.5, 0, NA)))
  expect_true(is.na(dunif(NA, 0, 1)))

  expect_true(is.na(dtriangle(0.5, NA, 1, 0.5)))
  expect_true(is.na(dtriangle(0.5, 0, NA, 0.5)))
  expect_true(is.na(dtriangle(0.5, 0, 1, NA)))
  expect_true(is.na(dtriangle(NA, 0, 1, 0.5)))

  # out of order
  expect_warning(expect_true(all(is.nan(dunif(0.5, 1, 0)))))
  # a > c
  expect_warning(expect_true(all(is.nan(dtriangle(0.5, 5, 6, 4)))))
  # b < c
  expect_warning(expect_true(all(is.nan(dtriangle(0.5, 5, 6, 7)))))

  # Inf
  expect_equal(dunif(1, -Inf, 2), 0)
  expect_equal(dunif(1, 0, Inf), 0)
  expect_equal(dunif(1, -Inf, Inf), 0)
  expect_equal(dunif(Inf, 0, 1), 0)

  expect_equal(dtriangle(.5, 0, Inf, 3), 0)
  expect_equal(dtriangle(.5, -Inf, 5, 3), 0)
  expect_equal(dtriangle(.5, 0, Inf, Inf), 0)
  expect_equal(dtriangle(Inf, 0, 1, 0.5), 0)
  expect_equal(dtriangle(-Inf, 0, 1, 0.5), 0)

  # multi-parameters
  expect_equal(dunif(c(0, 4, -1, 2), 0, 4), c(0.25, 0.25, 0, 0.25))
  expect_equal(dtriangle(c(1, 3, -1, .5), c(1, 0, -1, 0), c(3, 3, 3, 1),
                         c(2, 2, 2, .5)), c(0, 0, 0, 2))
})

test_that("dtriangle is not consistent with dunif", {
  # dunif recycles arguments even in non-recyclable lengths
  expect_error(dtriangle(c(1,2,3), c(1,2,3), c(4,5), c(1,2,3)))
})

# Copyright 2018 Rob Carnell

#' The Log-Triangle Distribution
#'
#' @description These functions provide information about the triangle distribution on the
#' logarithmic interval from \code{a} to \code{b} with a maximum at \code{c}.  \code{dltriangle}
#' gives the density, \code{pltriangle} gives the distribution function,
#' \code{qltriangle} gives the quantile function, and \code{rltriangle} generates
#' \code{n} random deviates.
#'
#' @details  All probabilities are lower tailed probabilties.  \code{a},
#' \code{b}, and \code{c} may be appropriate length vectors except in the
#' case of \code{rtriangle}.
#'
#' @param x,q vector of quantiles.
#' @param a lower limit of the distribution.
#' @param b upper limit of the distribution.
#' @param c mode of the distribution.
#' @param p vector of probabilities.
#' @param n number of observations. If \code{length(n) > 1}, the length is taken to be the number required.
#' @param logbase the base of the logarithmic scale to use (default to 10)
#'
#' @return   \code{dltriangle} gives the density, \code{pltriangle} gives the
#' distribution function, \code{qltriangle} gives the quantile function, and
#' \code{rltraingle} generates random deviates.  Invalid arguments will
#' result in return value \code{NaN} or \code{NA}.
#' @references
#' Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988) \emph{The New S Language}.  Wadsworth & Brooks/Cole.
#' @seealso
#' \code{\link{.Random.seed}} about random number generation,
#' \code{\link{runif}}, etc for other distributions.
#' @keywords distribution
#'
#' @name ltriangle
#' @importFrom stats runif
#' @export
#'
#' @examples
#' tri <- rltriangle(100000, 1, 100, 10)
#' hist(log10(tri), breaks=100, main="Triangle Distribution", xlab="x")
#' dltriangle(10, 1, 100, 10) # 2/(log10(b)-log10(a)) = 1
#' qltriangle(pltriangle(10)) # 10
rltriangle <- function(n=1, a=1, b=100, c=10^((log10(a) + log10(b))/2), logbase=10)
{
  stopifnot(length(n) == 1)
  if (n < 1 | is.na(n)) stop(paste("invalid argument: n =", n))
  n <- floor(n)
  if (any(is.na(c(a,b,c)))) return(rep(NaN, times = n)) # to match behavior of runif
  if (any(a > c | b < c)) return(rep(NaN, times = n)) # to match behavior of runif
  if (any(is.infinite(c(a, b, c)))) return(rep(NaN, times = n))
  if (any(c(a,b,c) == 0)) return(rep(-Inf, times = n))
  if (any(c(a,b,c) < 0)) return(rep(NaN, times = n))

  lp <- runif(n)

  stopifnot(length(logbase) == 1)
  if (logbase == 10)
  {
    la <- log10(a)
    lb <- log10(b)
    lc <- log10(c)
  } else
  {
    la <- log(a)/log(logbase)
    lb <- log(b)/log(logbase)
    lc <- log(c)/log(logbase)
  }

  if (a != c)
  {
    # if a = c then i is always true
    i <- which((la + sqrt(lp * (lb - la)*(lc - la))) <= lc)
    j <- which((lb - sqrt((1 - lp) * (lb - la) * (lb - lc))) > lc)
  } else
  {
    i <- which((la + sqrt(lp * (lb - la)*(lc - la))) < lc)
    j <- which((lb - sqrt((1 - lp) * (lb - la) * (lb - lc))) >= lc)
  }

  if (length(i) != 0)
    lp[i] <- la + sqrt(lp[i] * (lb - la) * (lc - la))
  if (length(j) != 0)
    lp[j] <- lb - sqrt((1 - lp[j]) * (lb - la) * (lb - lc))

  p <- logbase^lp

  return(p)
}

#' @rdname ltriangle
#' @export
dltriangle <- function(x, a=1, b=100, c=10^((log10(a) + log10(b))/2), logbase=10) {
  x1 <- length(x)
  a1 <- length(a)
  b1 <- length(b)
  c1 <- length(c)

  stopifnot(length(logbase) == 1)
  if (logbase == 10)
  {
    la <- log10(a)
    lb <- log10(b)
    lc <- log10(c)
    lx <- log10(x)
  } else
  {
    la <- log(a)/log(logbase)
    lb <- log(b)/log(logbase)
    lc <- log(c)/log(logbase)
    lx <- log(x)/log(logbase)
  }

  dTest <- function(X){
    if (any(is.na(X)))
    {
      # is.na is TRUE for NA, NaN, and FALSE
      if (any(is.nan(X))) return(NaN) # to conform to qunif
      else return(NA) # to conform to qunif
    } else if (X[2] > X[4] | X[3] < X[4] | (X[1] == X[2] & X[2] == X[4]))
    {
      warning("values required to be  a <= c <= b (at least one strict inequality)")
      return(NaN) # to conform to behavior of qunif
    } else if (any(is.infinite(X[2:4])))
    {
      return(NaN)
    } else if (X[1] <= X[2])
    {
      return(0)
    } else if (X[2] != X[4] & X[1] < X[4])
    {
      return(2*(X[1] - X[2]) / (X[3] - X[2]) / (X[4] - X[2]))
    } else if (X[4] != X[3] & X[1] >= X[4] & X[1] < X[3])
    {
      return(2*(X[3] - X[1]) / (X[3] - X[2]) / (X[3] - X[4]))
    } else if (X[1] >= X[3]) {
      return(0)
    }
  }

  k <- max(x1, a1, b1, c1)
  if (k == 1) return(dTest(c(lx, la, lb, lc)))

  params <- matrix(nrow = k, ncol = 4)
  tryCatch(
  {
    params[,1] <- lx
    params[,2] <- la
    params[,3] <- lb
    params[,4] <- lc
  }, error = function(X) {
    stop(paste(" -- Argument Lengths: length of x = ", x1,
                ", a = ", a1, ", b = ", b1, ", c = ", c1, " -- ", X, sep = ""))
  })

  return(apply(params, 1, dTest))
}

#' @rdname ltriangle
#' @export
pltriangle <- function(q, a=1, b=100, c=10^((log10(a) + log10(b))/2), logbase=10)
{
  q1 <- length(q)
  a1 <- length(a)
  b1 <- length(b)
  c1 <- length(c)

  stopifnot(length(logbase) == 1)
  if (logbase == 10)
  {
    la <- log10(a)
    lb <- log10(b)
    lc <- log10(c)
    lq <- log10(q)
  } else
  {
    la <- log(a)/log(logbase)
    lb <- log(b)/log(logbase)
    lc <- log(c)/log(logbase)
    lq <- log(q)/log(logbase)
  }

  pTest <- function(X)
  {
    if (any(is.na(X))) { # is.na is TRUE for NA, NaN, and FALSE
      if (any(is.nan(X))) return(NaN) # to conform to qunif
      else return(NA) # to conform to qunif
    } else if (X[2] > X[4] | X[3] < X[4] | (X[1] == X[2] & X[2] == X[4]))
    {
      warning("values required to be  a <= c <= b (at least one strict inequality)")
      return(NaN) # to conform to behavior of qunif
    } else if (any(is.infinite(X[2:4])))
    {
      return(NaN)
    } else if (X[1] <= X[2])
    {
      return(0)
    } else if (X[2] != X[4] & X[1] < X[4])
    {
      return((X[1] - X[2])^2 / (X[3] - X[2]) / (X[4] - X[2]))
    } else if (X[4] != X[3] & X[1] >= X[4] & X[1] < X[3])
    {
      return(1 - (X[3] - X[1])^2 / (X[3] - X[2]) / (X[3] - X[4]))
    } else if (X[1] >= X[3])
    {
      return(1)
    }
  }

  k <- max(q1, a1, b1, c1)
  if (k == 1) return(pTest(c(lq, la, lb, lc)))

  params <- matrix(nrow = k, ncol = 4)
  tryCatch(
  {
    params[,1] <- lq
    params[,2] <- la
    params[,3] <- lb
    params[,4] <- lc
  }, error = function(X) {
    stop(paste(" -- Argument Lengths: length of q = ", q1,
                ", a = ", a1, ", b = ", b1, ", c = ", c1, " -- ", X, sep = ""))
  })

  return(apply(params, 1, pTest))
}

#' @rdname ltriangle
#' @export
qltriangle <- function(p, a=1, b=100, c=10^((log10(a) + log10(b))/2), logbase=10)
{
  p1 <- length(p)
  a1 <- length(a)
  b1 <- length(b)
  c1 <- length(c)

  stopifnot(length(logbase) == 1)
  if (logbase == 10)
  {
    la <- log10(a)
    lb <- log10(b)
    lc <- log10(c)
  } else
  {
    la <- log(a)/log(logbase)
    lb <- log(b)/log(logbase)
    lc <- log(c)/log(logbase)
  }

  qTest <- function(X)
  {
    # X = c(p, a, b, c)
    if (any(is.na(X))) { # is.na is TRUE for NA, NaN, and FALSE
      if (any(is.nan(X))) return(NaN) # to conform to qunif
      else return(NA) # to conform to qunif
    } else if (X[2] > X[4] | X[3] < X[4])
    {
      warning("values required to be  a <= c <= b (at least one strict inequality)")
      return(NaN) # to conform to behavior of qunif
    } else if (X[1] < 0 | X[1] > 1)
    {
      warning("at least one p is outside [0,1]")
      return(NaN) # to conform to behavior of qunif
    } else if (any(is.infinite(X)))
    {
      return(NaN)
    } else if ((X[2] != X[4] &&
               (X[2] + sqrt(X[1]*(X[3] - X[2])*(X[4] - X[2]))) <= X[4]) |
              (X[2] == X[4] &&
               (X[2] + sqrt(X[1]*(X[3] - X[2])*(X[4] - X[2]))) < X[4]))
    {
      return(logbase^(X[2] + sqrt(X[1]*(X[3] - X[2])*(X[4] - X[2]))))
    } else if ((X[2] != X[4] &&
               (X[3] - sqrt((1 - X[1])*(X[3] - X[2])*(X[3] - X[4]))) > X[4]) |
              (X[2] == X[4] &&
               (X[3] - sqrt((1 - X[1])*(X[3] - X[2])*(X[3] - X[4]))) >= X[4]))
    {
      return(logbase^(X[3] - sqrt((1 - X[1])*(X[3] - X[2])*(X[3] - X[4]))))
    } else if (X[2] != X[4] && (X[2] + sqrt(X[1]*(X[3] - X[2])*(X[4] - X[2]))) - X[4] <= (X[3] - X[2])*.Machine$double.eps)
    {
      return(X[4])
    } else stop("Unexpected Result")
  }

  k <- max(p1, a1, b1, c1)
  if (k == 1) return(qTest(c(p,la,lb,lc)))

  params <- matrix(nrow = k, ncol = 4)
  tryCatch(
  {
    params[,1] <- p
    params[,2] <- la
    params[,3] <- lb
    params[,4] <- lc
  }, error = function(X) {
    stop(paste(" -- Argument Lengths: length of p = ", p1,
                ", a = ", a1, ", b = ", b1, ", c = ", c1, " -- ", X, sep = ""))
  })

  return(apply(params, 1, qTest))
}

# Copyright 2026 Rob Carnell

test_that("rltriangle",
{
  expect_true(max(range(rltriangle(100))) <= 100)
  expect_true(min(range(rltriangle(100))) >= 1)
  expect_true(length(rltriangle(20.3)) == 20)

  expect_error(rltriangle(-5))
  expect_equal(rltriangle(2, NaN, 3, 1), c(NaN, NaN))
  expect_equal(rltriangle(1, 3, NA, 4), NaN)
  expect_error(rltriangle(NA))

  test <- rltriangle(1, 2, 5, 2 + 10^-6)
  expect_true(test > 2 & test < 5)
  test <- rltriangle(1, 2, 5, 2 + 10^-9)
  expect_true(test > 2 & test < 5)
  test <- rltriangle(1, 2, 5, 2 + 10^-12)
  expect_true(test > 2 & test < 5)
  expect_true(!is.nan(rltriangle(1, 2, 5, 2 + 10^-18)))

  expect_true(!all(1 == rltriangle(100, 1, 100, 1)))
  expect_true(!any(is.nan(rltriangle(100, 10, 100, 10))))
  expect_true(all(!is.finite(rltriangle(10, -1, 0, -1))))
  expect_true(all(is.na(rltriangle(10, -5, -2, -2))))
  expect_true(all(1 == rltriangle(10,1,1,1)))
  expect_true(all(is.na(rltriangle(10,-1,-1,-1))))

  expect_true(all(is.nan(rltriangle(10, 5, 6, 4))))
  expect_true(all(is.nan(rltriangle(10, 5, 6, 7))))
  expect_true(all(is.nan(rltriangle(10, 5, Inf, 7))))

  expect_true(all(rltriangle(3, 1, 100, 10, logbase = 7) >= 1))
})

test_that("dltriangle",
{
  expect_equal(dltriangle(1, 1, 3, 2), 0)
  expect_equal(dltriangle(3, 1, 3, 2), 0)
  expect_equal(dltriangle(10^0.5, 1, 10, 10^0.5), 2)
  expect_warning(expect_equal(dltriangle(0, 0, 0, 0), NaN))
  expect_warning(expect_equal(dltriangle(.5, 0, 1, 2), NaN))
  expect_warning(expect_equal(dltriangle(.5, 1, 2, .5), NaN))
  expect_equal(dltriangle(c(1, 3, 10^.5), c(1, 1, 1), c(3, 3, 10),
                                c(2, 2, 10^.5)), c(0, 0, 2))

  expect_equal(dltriangle(7, 1, 49, 7, logbase = 7), 1)

  expect_true(is.na(dltriangle(.5, 0, NA, 3)))
  expect_true(is.nan(dltriangle(.5, 0, 5, NaN)))
  expect_true(is.nan(dltriangle(.5, 0, Inf, 3)))

  expect_error(dltriangle(c(1,2,3), c(1,2,3), c(4,5), c(1,2,3)))

  expect_equal(0.5, dltriangle(10^0.5, 1, 100, 10))
})

test_that("ptriangle",
{
  expect_equal(pltriangle(1, 1, 3, 2), 0)
  expect_equal(pltriangle(3, 1, 3, 2), 1)
  expect_equal(pltriangle(10^.5, 1, 10, 10^.5), .5)
  expect_warning(expect_equal(pltriangle(0, 0, 0, 0), NaN))
  expect_warning(expect_equal(pltriangle(.5, 0, 1, 2), NaN))
  expect_warning(expect_equal(pltriangle(.5, 1, 2, .5), NaN))
  expect_equal(pltriangle(c(1, 3, 10^.5), c(1, 1, 1), c(3, 3, 10),
                                c(2, 2, 10^.5)), c(0, 1, .5))

  expect_equal(pltriangle(7, 1, 49, 7, logbase = 7), 0.5)

  expect_true(is.na(pltriangle(1, 0, NA, 3)))
  expect_true(is.nan(pltriangle(1, 0, 5, NaN)))
  expect_true(is.nan(pltriangle(1, 0, Inf, 3)))

  expect_error(pltriangle(c(1,2,3), c(1,2,3), c(4,5), c(1,2,3)))
})

test_that("qltriangle",
{
  expect_equal(qltriangle(0, 1, 3, 2), 1)
  expect_equal(qltriangle(0, 2, 3, 2), 2)
  expect_equal(qltriangle(1, 1, 3, 2), 3)
  expect_equal(qltriangle(.5, 1, 10, 10^.5), 10^.5)
  expect_equal(qltriangle(0, 1, 1, 1), 1)
  expect_warning(expect_equal(qltriangle(-1, 0, 1, .5), NaN))
  expect_warning(expect_equal(qltriangle(2, 0, 1, .5), NaN))
  expect_warning(expect_equal(qltriangle(.5, 0, 1, 2), NaN))
  expect_warning(expect_equal(qltriangle(.5, 1, 2, .5), NaN))
  expect_equal(qltriangle(.5, NA, 2, .5), NA)
  expect_equal(qltriangle(c(0, 0, 1, .5), c(1, 2, 1, 1),
                                c(3, 3, 3, 10), c(2, 2, 2, 10^.5)),
                     c(1, 2, 3, 10^.5))
  expect_error(qltriangle(c(0, 0, 0, 0, 0), c(0, 0), 1, .5))
  expect_equal(qltriangle(.5, NaN, 2, 1), NaN)
  expect_equal(qltriangle(.5, 0, Inf, 1), NaN)
  expect_equal(qltriangle(.5, 0, Inf, Inf), NaN)

  expect_equal(qltriangle(pltriangle(15, 1, 100, 10), 1, 100, 10), 15)
  expect_equal(qltriangle(pltriangle(3, 2, 5, 5), 2, 5, 5), 3)
  expect_equal(qltriangle(pltriangle(5, 2, 5, 5), 2, 5, 5), 5)

  expect_equal(qltriangle(0.5, 1, 49, 7, logbase = 7), 7)

  expect_true(is.na(qltriangle(0.5, 0, NA, 3)))
  expect_true(is.nan(qltriangle(0.5, 0, 5, NaN)))
  expect_true(is.nan(qltriangle(0.5, 0, Inf, 3)))

  expect_error(qltriangle(c(.1,.2,.3), c(1,2,3), c(4,5), c(1,2,3)))

  # numerical stability issue
  expect_equal(3, qltriangle(0 + 0.1 + 0.1 + 0.1, 10^0, 10^10, 10^3))
})

# Copyright Robert Carnell 2022

#' Triangle parameter estimates using a non-linear fit of the empirical CDF
#'
#' @param x the triangle distributed sample
#' @param control an object created by \code{stats::nls.control}
#'
#' @return an object of class \code{nls}
#' @export
#'
#' @importFrom stats nls nls.control
#'
#' @examples
#' set.seed(10304)
#' xtest <- rtriangle(100, 1, 5, 2)
#' cdfe <- triangle_cdfe(xtest)
#' print(cdfe)
#' summary(cdfe)
#' coef(cdfe)
#' \dontrun{
#'   confint(cdfe)
#' }
triangle_cdfe <- function(x, control = stats::nls.control(maxiter = 100, warnOnly = TRUE))
{
  # x <- rtriangle(100, 0, 1, .3)
  n <- length(x)
  ecdf1 <- sapply(1:n, function(i) length(which(x < x[i]))) / n
  minx <- min(x)
  maxx <- max(x)
  rangex <- maxx - minx

  nls1 <- stats::nls(
    ecdf1 ~ ptriangle(x, a, b, c),
    start = c(a = minx - 0.5*rangex,
              b = maxx + 0.5*rangex,
              c = 3*mean(x) - minx - maxx),
    algorithm = "port",
    lower = c(minx - 2*rangex, maxx, minx),
    upper = c(minx, maxx + 2*rangex, maxx),
    control = control)
  return(nls1)
}

---
title: "Triangle Distribution Math"
author: "Rob Carnell"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Triangle Distribution Math}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{rmarkdown, knitr}
header-includes:
  - \usepackage{amsmath}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Triangle Notation

```{r triangle_drawing, echo=FALSE, fig.width=5, fig.height=4}
plot(c(-1,5), c(0,4), type = "n", xlab = "x", ylab = "f(x)", axes = FALSE)
lines(c(0,1), c(0,3), col = "red")
lines(c(1,4), c(3,0), col = "red")
axis(2, at = c(0,3), labels = c("0","h"), las = 2)
axis(1, at = c(0,1,4), labels = c("a", "c", "b"))
box()
```

- $a$ = minimum
- $b$ = maximum
- $c$ = mode
- $h$ = density at the mode = $\frac{2}{b-a}$

## Triangle Probability Density Funciton (PDF)

**Lemma 1: (Triangle PDF).**

*Given, * $x, a, b, c  \in \mathbb{R}$ *and* $a \le c \lt b$ *or* $a \lt c \le b$ 
*the triangle probability density function is given by*

$$
\begin{equation}
f(x) = \begin{cases}
		\frac{2}{(b-a)(c-a)}(x-a)  & \mbox{if } a \leq x \leq c \\
		\frac{2}{(b-a)(c-b)}(x-b) & \mbox{if } c < x \leq b \\
		0 & \mbox{otherwise}
\end{cases}
\end{equation}
$$

*if a random variable X has a PDF f(x), then we say* $X \sim triangle(a, b, c)$

**Proof**

The triangle is made of two lines:

$$(y-0) = \frac{h-0}{c-a}(x-a)$$

$$(y-0)=\frac{h-0}{c-b}(x-b)$$


Integrating the pdf under these two equations to solve for $h$:

$$\int f(x) dx = 1$$

$$\frac{h}{c-a}\int_{a}^{c} (x-a) dx + \frac{h}{c-b} \int_{c}^{b} (x-b) dx = \frac{h(b-a)}{2} = 1$$

$$h=\frac{2}{b-a}$$

The PDF should be zero at each end of the interval and be continuous at $c$

$$f(a) = \frac{2}{(b-a)(c-a)}(a-a) = 0$$

$$f(b) = \frac{2}{(b-a)(c-b)}(b-b)$$

$$f_{a \le x \le c}(c) = \frac{2}{(b-a)(c-a)}(c-a) = \frac{2}{(b-a)}$$

$$\lim_{x \to c} f_{c \lt x \le b}(x) = \lim_{x \to c} \frac{2}{(b-a)(c-b)}(x-b) = \frac{2}{(b-a)}$$

## Triangle Cumulative Distribution Function (CDF)

**Lemma 2: (Triangle CDF).**

*Given, * $x, a, b, c  \in \mathbb{R}$ *and* $a \le c \lt b$ *or* $a \lt c \le b$ 
*the cumulative distribution function over* $x$ *is*

$$
\begin{equation}
F(x) = \begin{cases}
    0 & \mbox{if } x < a \\
		\frac{(x-a)^2}{(b-a)(c-a)}  & \mbox{if } a \leq x \leq c \\
		1 + \frac{(x-b)^2}{(b-a)(c-b)} & \mbox{if } c < x \leq b \\
		1 & \mbox{if } x > b
\end{cases}
\end{equation}
$$

**Proof**

$$F(x) = \int_{-\infty}^{x} f(t) dt$$

$$
\begin{align}
F_{a \le x \le c}(x) &= \int_a^x \frac{2(t-a)}{(b-a)(c-a)} dt \\
&= \frac{(x-a)^2}{(b-a)(c-a)}
\end{align}
$$

$$
\begin{align}
F_{c \lt x \lt b}(x) &= F_{a \le x \le c}(c) + \int_c^x \frac{2(t-b)}{(b-a)(c-b)} dt = 1 - \int_x^b \frac{2(t-b)}{(b-a)(c-b)} dt \\
&= 1 + \frac{(x - b)^2}{(b-a)(c-b)} = 1 - \frac{(b - x)^2}{(b-a)(b - c)}
\end{align}
$$

The CDF should be zero at $a$, continuous at $c$, and one at $b$

$$F_{a \le x \le c}(a) = \frac{(a-a)^2}{(b-a)(c-a)} = 0$$

$$F_{c \lt x \lt b}(b) = 1 - \frac{(b - b)^2}{(b-a)(b - c)} = 1$$

$$F_{a \le x \le c}(c) = \frac{(c-a)^2}{(b-a)(c-a)} = \frac{(c-a)}{(b-a)}$$

$$\lim_{x \to c} F_{c \lt x \lt b}(c) = \lim_{x \to c} \left[ 1 - \frac{(b - x)^2}{(b-a)(b - c)} \right] = 1 - \frac{(b - c)}{(b-a)} = \frac{(c-a)}{(b-a)}$$

## Triangle Mean

**Lemma 3: (Triangle Mean).**

*The mean of the triangle distribution is*

$$E(X) = \frac{a+b+c}{3}$$

**Proof**

$$
\begin{align}
E(X) &= \int xf(x)dx = \frac{2}{(b-a)(c-a)}\int_a^c (x^2-ax) dx + 
\frac{2}{(b-a)(c-b)}\int_c^b (x^2-bx) dx \\
&= \frac{2}{(b-a)(c-a)} \left[\frac{1}{3}x^3-\frac{a}{2}x^2\right]_a^c + \frac{2}{(b-a)(c-b)}\left[\frac{1}{3}x^2-\frac{b}{2}x^2\right]_c^b dx \\
&= \frac{a+b+c}{3}
\end{align}
$$

## Triangle Variance

**Lemma 4: (Triangle Variance).**

*The variance of the triangle distribution is*

$$V(X) = \frac{a^2+b^2+c^2-ab-ac-bc}{18}$$

**Proof**

$$
\begin{align}
V(X) &= E(X^2) - \big(E(X)\big)^2 = \int x^2f(x)dx- \bigg(\frac{a+b+c}{3}\bigg)^2 \\
&= \frac{2}{(b-a)(c-a)}\int_{a}^{c} x^2(x-a) dx + \frac{2}{(b-a)(c-b)} \int_{c}^{b} x^2(x-b) dx- \bigg(\frac{a+b+c}{3}\bigg)^2 \\
&= \frac{2}{(b-a)(c-a)} \left[\frac{1}{4}x^4-\frac{a}{3}x^3\right]_a^c + \frac{2}{(b-a)(c-b)}\left[\frac{1}{4}x^4-\frac{b}{3}x^3\right]_c^b - \bigg(\frac{a+b+c}{3}\bigg)^2 \\
&= \frac{a^2+b^2+c^2-ab-ac-bc}{18}
\end{align}
$$

## Method of Moments Estimation

The Type 1 and 2 notation is used in the package, but are not externally accepted "types" of
Methods of Moments.

### Type 1

**Lemma 5: (Method of Moments Estimator 1).**

*Estimators for the Triangle parameters are*

$$\hat{a} = min(x) = X_{(1)}$$

$$\hat{b} = max(x) = X_{(n)}$$

$$\hat{c} = 3\bar{x} - \min(x) - \max(x)$$

**Motivation**

$$E(X) = \frac{a + b + c}{3}$$

$$c = 3E(X) - a - b$$

The sample minimum is an overestimate for $a$ and the sample maximum is an 
underestimate for $b$.

$\hat{c}$ is a biased estimator for $c$.  

$$
\begin{align}
E(\hat{c}) =& 3E(\bar{x}) - E(X_{(1)}) - E(X_{(n)}) \\
=& \frac{3}{n} \sum_{i=1}^n E(X_i) \\
 &- n \left[\sum_{k=0}^{n-1} {n-1 \choose k} \left(\frac{c-a}{b-a}\right)^{n-k} (-1)^{n-1-k} \frac{2c(n-k)+a}{(n-k)(2(n-k) + 1)} - \left(\frac{b-c}{b-a}\right)^n \frac{2cn+b}{n(2n+1)} \right] \\
 &- n \left[ \left(\frac{c-a}{b-a}\right)^n \frac{2cn+a}{n(2n + 1)} + \sum_{k=0}^{n-1} {n-1 \choose k} \left(\frac{c-b}{b-a}\right)^{n-k} \frac{2c(n-k)+b}{(n-k)(2(n-k)+1)} \right] \\
=& a + b + c - [expansion] - [expansion]\\
\neq & c
\end{align}
$$

Simulation shows that $\hat{c}$ is consistent for $c$.

### Type 2

**Lemma 6: (Method of Moments Estimator 2).**

*Estimators for the Triangle parameters are the solution to these equations for the mean, variance, and skewness*

$$\bar{x} = \frac{1}{n} \sum_i x_i = \frac{\hat{a} + \hat{b} + \hat{c}}{3}$$

$$\frac{1}{n-1} \sum_i (x_i - \bar{x})^2 = \frac{\hat{a}^2+\hat{b}^2+\hat{c}^2-\hat{a}\hat{b}-\hat{a}\hat{c}-\hat{b}\hat{c}}{18}$$

$$\frac{\sqrt{n} \sum_i (x_i-\bar{x})^3}{\left[\sum_i (x_i-\bar{x})^2\right]^{3/2}} = \frac{\sqrt{2} (\hat{a} + \hat{b} - 2\hat{c}) (2\hat{a} - \hat{b} - \hat{c}) (\hat{a} - 2\hat{b} + \hat{c})}{5(\hat{a}^2+\hat{b}^2+\hat{c}^2-\hat{a}\hat{b}-\hat{a}\hat{c}-\hat{b}\hat{c})^{3/2}}$$


## Maximum Likelihood Estimation

The procedure for maximum likelihood estimation involves maximizing the 
likelihood with respect to $c$ for a fixed $a$ and $b$,
followed by minimizing the negative log likelihood with respect to $a$ and $b$
for a fixed $c$.  

###  Maximizing the Likelihood with respect to $c$ (given $a$ and $b$)

This discussion follows the results from [Samuel Kotz and Johan Rene van Dorp. Beyond Beta](https://doi.org/10.1142/5720)

For the purposes of this section, with a fixed $a$ and $b$, the sample can be easily
rescaled to $a=0$ and $b=1$.  This section will proceed on $[0,1]$ with the mode at $0 \le c \le 1$

$$w(x) = 
\left\{
  \begin{array}{ll}
    \frac{2x}{c} & \mbox{if } 0 \le x \lt c \\
    \frac{2(1-x)}{1-c} & \mbox{if } c \le x \leq 1 \\
    0 & \mbox{otherwise}
  \end{array}
\right.$$

$$L(x|c) = \prod_{i}^{n} w(x|c)$$

Assume that the sample is ordered into order statistics $X_{(1)} \lt \dots \lt X_{(n)}$.  Also, note
that $X_{(r)} \le c \lt X_{(r+1)}$.  In other words, the mode falls between the $r^{th}$ and $r+1$ order statistics.

$$L(x|c) = \prod_{i=1}^{r} \frac{2x_{(i)}}{c} \prod_{i=r+1}^{n} \frac{2(1-x_{(i)})}{1-c} = \frac{2^n \prod_{i=1}^{r} x_{(i)} \prod_{i=r+1}^{n} (1-x_{(i)})}{c^r(1-c)^{n-r}}$$

To maximize the likelihood, we can first maximize with respect to $r$ and then locate $c$ between
the $r^{th}$ and $r+1$ order statistics.  For notation purposes, also define $X_{(0)} = 0$ and $X_{(n+1)} = 1$.

$$\large \max_{0 \le c \le 1} L(x|c) = \max_{r \ \epsilon \ (0,\dots,n)} \ \ \max_{x_{(r)} \le c \le x_{(r+1)}} \ \ L(x|c)$$

#### Case 1:  $c$ is between the first and second to last order statistic $r \ \epsilon \ (1, \dots, n-1)$

Noticing that maximizing the likelihood is equivalent to minimizing the denominator:

$$\large \max L(x|c) = \max_{r \ \epsilon \ (1,\dots,n-1)} \ \ \min_{x_{(r)} \le c \le x_{(r+1)}} \ \ c^r(1-c)^{n-r}$$

Since $c^r(1-c)^{n-r}$ is unimodal with respect to $c$, it should be sufficient to test the end points
of an interval to find the minimum on the interval

$$\large = \max_{r \ \epsilon \ (1,\dots,n-1)} \ \ \min_{c \ \epsilon \ (x_{(r)},\ \ x_{(r+1)})} \ \ c^r(1-c)^{n-r}$$

Therefore, for this case, it is sufficient to test the likelihood using $c$ at each of the sampled points and find the largest.

##### Side note on $z=c^r(1-c)^{n-r}$ being unimodal

$$\frac{dz}{dc} = rc^{(r-1)}(1-c)^{n-r} + c^r(n-r)(1-c)^{n-r-1}(-1) = c^{(r-1)}(1-c)^{n-r-1}(r - cn)$$

$\frac{dz}{dc} = 0$ at $c=0,\ 1,\ \frac{r}{n}$.  At $0 < c < \frac{r}{n}$, $z$ is positive, and at $\frac{r}{n} < c < 1$, $z$ is negative.
Therefore, $z$ is unimodal on $(0,1)$.

#### Case 2:  $c$ is between 0 and the first order statistic $r = 0$

$$\large \max L(x|c) = \max_{0 \le c \le x_{(1)}} \prod_{i=1}^{n} \frac{1-x_{(i)}}{1-c} = \prod_{i=1}^{n} \frac{1-x_{(i)}}{1-x_{(1)}}$$

Choosing the largest endpoint in the interval, creates the smallest denominator, and the largest likelihood.

Therefore, for this case, it is sufficient to test the likelihood using $c$ at the first sampled point.

#### Case 3:  $c$ is between the last order statistic $r = n$ and 1

$$\large \max L(x|c) = \max_{x_{(n)} \le c \le 1} \prod_{i=1}^{n} \frac{x_{(i)}}{c} = \prod_{i=1}^{n} \frac{x_{(i)}}{x_{(n)}}$$

Choosing the smallest option in the denominator creates the largest likelihood.  Again, it is sufficient
to test the likelihood using $c$ at the largest sample point.

#### All Cases

For all cases, it is sufficient to compute the sample likelihood using $c$ equal
to each of the samples, and choosing the largest likelihood from the $n$ options
to find the corresponding $c$.  This calculation is performed with a fixed $a$ and $b$,
so the test must be performed iteratively as $a$ and $b$ are separately optimized.

### Negative Log Likelihood

$$
\begin{align}
nLL &= -\log(L) = -\log\left(\prod_i^n f(x_i)\right) \\
&= - \sum_i^n \log\left(f(x_i)\right) = - \sum_{i: \ a \le x_i \lt c}^{n_1}  \log\left(f(x_i)\right) - \sum_{i: \ c \le x_i \le b}^{n_2} \log\left(f(x_i)\right)
\end{align}
$$

where $n = n_1 + n_2$

#### Case 1:  $a = c \lt b$

$$
\begin{align}
nLL &= - \sum_{i}^{n} \log(2) + \log(b-x_i) - \log(b-a) - \log(b-c) \\
&= -n\log(2) + n\log(b-a) + n \log(b-c) - \sum_{i}^{n} \log(b-x_i)
\end{align}
$$

#### Case 2:  $a \lt c = b$

$$
\begin{align}
nLL &= - \sum_{i}^{n} \log(2) + \log(x_i - a) - \log(b-a) - \log(c-a) \\
&= -n\log(2) + n\log(b-a) + n\log(c-a) - \sum_{i}^{n} \log(x_i - a)
\end{align}
$$

#### Case 3:  $a \lt c \lt b$

$$
\begin{align}
nLL &= - \sum_{i: \ a \lt x_i \lt c}^{n_1} \log(2) + \log(x_i - a) - \log(b-a) - \log(c-a) - \sum_{i: \ c \le x_i \lt b}^{n_2} \log(2) + \log(b-x_i) - \log(b-a) - \log(b-c) \\
&= -n\log(2) + n\log(b-a) + n_1\log(c-a) + n_2 \log(b-c) - \sum_{i: \ a \lt x_i \lt c}^{n_1} \log(x_i - a) - \sum_{i: \ c \le x_i \lt b}^{n_2} \log(b-x_i)
\end{align}
$$

### Gradient of the negative Log Likelihood Given $c$:

The negative log likelihood is not differentiable with respect to $c$ because the limits of the sum ($n_1$ and $n_2$) are
functions of $c$.  Therefore the gradient and hessian are derived as if $c$ is fixed.

#### Case 1:  $a = c \lt b$

$$\frac{\partial nLL}{\partial a} = - \frac{n}{b-a}$$

$$\frac{\partial nLL}{\partial b} = \frac{n}{b-a} + \frac{n}{b-c} - \sum_i^{n} \frac{1}{b-x_i}$$

#### Case 2:  $a \lt c = b$

$$\frac{\partial nLL}{\partial a} = - \frac{n}{b-a} - \frac{n}{c-a} + \sum_i^{n} \frac{1}{x_i - a}$$

$$\frac{\partial nLL}{\partial b} = \frac{n}{b-a}$$

#### Case 3:  $a \lt c \lt b$

$$\frac{\partial nLL}{\partial a} = - \frac{n}{b-a} - \frac{n_1}{c-a} + \sum_i^{n_1} \frac{1}{x_i - a}$$

$$\frac{\partial nLL}{\partial b} = \frac{n}{b-a} + \frac{n_2}{b-c} - \sum_i^{n_2} \frac{1}{b-x_i}$$

### Hessian of the negative Log Likelihood Given $c$:

#### Case 1:  $a = c \lt b$

$$\frac{\partial^2nLL}{\partial a^2} = - \frac{n}{(b-a)^2}$$

$$\frac{\partial^2 nLL}{\partial b^2} = -\frac{n}{(b-a)^2} - \frac{n}{(b-c)^2} + \sum_i^{n} \frac{1}{(b-x_i)^2}$$

$$\frac{\partial^2 nLL}{\partial a\partial b} = \frac{\partial^2 nLL}{\partial b\partial a} = - \frac{n}{(b-a)^2}$$

#### Case 2:  $a \lt c = b$

$$\frac{\partial^2 nLL}{\partial a^2} = - \frac{n}{(b-a)^2} - \frac{n}{(c-a)^2} + \sum_i^{n} \frac{1}{(x_i - a)^2}$$

$$\frac{\partial^2 nLL}{\partial b^2} = - \frac{n}{(b-a)^2}$$

$$\frac{\partial^2 nLL}{\partial a\partial b} = \frac{\partial^2 nLL}{\partial b\partial a} = - \frac{n}{(b-a)^2}$$

#### Case 3:  $a \lt c \lt b$

$$\frac{\partial^2 nLL}{\partial a^2} = - \frac{n}{(b-a)^2} - \frac{n_1}{(c-a)^2} + \sum_i^{n_1} \frac{1}{(x_i - a)^2}$$


$$\frac{\partial ^2 nLL}{\partial b^2} = -\frac{n}{(b-a)^2} - \frac{n_2}{(b-c)^2} + \sum_i^{n_2} \frac{1}{(b-x_i)^2}$$

$$\frac{\partial ^2 nLL}{\partial a\partial b} = \frac{\partial ^2 nLL}{\partial b\partial a} = - \frac{n}{(b-a)^2}$$

### MLE Variance - Covariance

For the optimization of $(a,b)$ given $c$, we can use the inverse of the hessian
of the negative log likelihood for an estimate of the covariance matrix of $\hat{a}$ 
and $\hat{b}$.  For the variance in $\hat{c}$, we use the variance of the $r^{th}$ order
statistic which corresponds to $c$.  The covariance of $(a,b)$ and $c$ is not
computed because the negative log likelihood is not differentiable with respect to $c$.

Let $H$ denote the Hessian matrix, and let $H^{-1}[1,1]$ be the $V(\hat{a})$,
$H^{-1}[2,2]$ be the $V(\hat{b})$, and $H^{-1}[1,2] = H^{-1}[2,1]$ be the 
$Cov(\hat{a}, \hat{b})$.  Then,

$$ V([\hat{a}, \hat{b}, \hat{c}]) =   
\begin{bmatrix} 
   H^{-1}[1,1] & H^{-1}[1,2] & 0  \\
   H^{-1}[2,1] & H^{-1}[2,2] & 0  \\
   0 & 0 & V(\hat{c})  \\
\end{bmatrix} 
$$

#### $r^{th}$ order statistic

$$f(x_{(r)}) = r {n \choose r} f(x) [F(x)]^{r-1}[1-F(x)]^{n-r}$$

#### Expected value of the $r^{th}$ order statistic

$$
\begin{align}
E(X_{(r)}) &= \int x f(x_{(r)}) dx \\
&= \int_a^c xr {n \choose r} \frac{2(x-a)}{(b-a)(c-a)} \left(\frac{(x-a)^2}{(b-a)(c-a)}\right)^{r-1}\left(1 - \frac{(x-a)^2}{(b-a)(c-a)}\right)^{n-r}dx \\
&+ \int_c^b xr {n \choose r} \frac{2(x-b)}{(b-a)(c-b)} \left(1+\frac{(x-b)^2}{(b-a)(c-b)}\right)^{r-1}\left(- \frac{(x-b)^2}{(b-a)(c-b)}\right)^{n-r}dx
\end{align}
$$

To simplify the notation, define:

$$\gamma_0 = 2r {n \choose r}$$

$$\gamma_1 = (b-a)(c-a)$$

$$\gamma_2 = (b-a)(c-b)$$

Continuing:

$$E(X_{(r)}) = \int_a^c \frac{\gamma_0}{\gamma_1^n} x(x-a)^{2r-1} \left(\gamma_1 - (x-a)^2\right)^{n-r}dx + \int_c^b \frac{\gamma_0}{\gamma_2^n} (-1)^{n-r}x(x-b)^{2n-2r+1} \left(\gamma_2 + (x-b)^2\right)^{r-1}dx$$

By using a binomial expansion, we can prevent having to integrate by parts multiple times.

$$(a+b)^n = \sum_{k=0}^n {n \choose k} a^kb^{n-k}$$

$$
\begin{align}
E(X_{(r)}) &= \int_a^c \frac{\gamma_0}{\gamma_1^n} x(x-a)^{2r-1} \sum_{k=0}^{n-r} {n-r \choose k} \gamma_1^k (-1)^{n-r-k}(x-a)^{2n-2r-2k}dx \\
&+ \int_c^b \frac{\gamma_0}{\gamma_2^n} (-1)^{n-r}x(x-b)^{2n-2r+1} \sum_{k=0}^{r-1} {r-1 \choose k} \gamma_2^k (x-b)^{2r-2-2k}dx
\end{align}
$$

$$
\begin{align}
E(X_{(r)}) &= \frac{\gamma_0}{\gamma_1^n} \sum_{k=0}^{n-r}   {n-r \choose k} \gamma_1^k (-1)^{n-r-k}\int_a^c x(x-a)^{2n-2k-1}dx \\
&+ \frac{\gamma_0}{\gamma_2^n} (-1)^{(n-r)} \sum_{k=0}^{r-1} {r-1 \choose k} \gamma_2^k \int_c^b x(x-b)^{2n-2k-1}dx
\end{align}
$$

$$
\begin{align}
E(X_{(r)}) &= \gamma_0 \sum_{k=0}^{n-r}   {n-r \choose k} \gamma_1^{k-n} (-1)^{n-r-k} \left[\frac{c(c-a)^{2n-2k}}{2n-2k} - \frac{(c-a)^{2n-2k+1}}{(2n-2k)(2n-2k+1)}\right] \\
&+ \gamma_0 (-1)^{n-r} \sum_{k=0}^{r-1} {r-1 \choose k} \gamma_2^{k-n}  \left[\frac{-c(c-b)^{2n-2k}}{2n-2k}+\frac{(c-b)^{2n-2k+1}}{(2n-2k)(2n-2k+1)}\right]
\end{align}
$$

$$
\begin{align}
E(X_{(r)}) &= r {n \choose r} \sum_{k=0}^{n-r}   {n-r \choose k} (b-a)^{k-n} (c-a)^{n-k} (-1)^{n-r-k} \left[\frac{c}{n-k} - \frac{c-a}{(n-k)(2n-2k+1)}\right] \\
&+ r {n \choose r} (-1)^{n-r} \sum_{k=0}^{r-1} {r-1 \choose k} (b-a)^{k-n} (c-b)^{n-k} \left[\frac{-c}{n-k}+\frac{c-b}{(n-k)(2n-2k+1)}\right]
\end{align}
$$

$$
\begin{align}
E(X_{(r)}) &= r {n \choose r}  \bigg[ \ \sum_{k=0}^{n-r}   {n-r \choose k} (b-a)^{k-n} (c-a)^{n-k} (-1)^{n-r-k} \frac{2c(n-k)+a}{(n-k)(2(n-k) + 1)} \\
&+ (-1)^{n-r+1} \sum_{k=0}^{r-1} {r-1 \choose k} (b-a)^{k-n} (c-b)^{n-k} \frac{2c(n-k)+b}{(n-k)(2(n-k)+1)} \ \bigg]
\end{align}
$$

#### Expected Value of $r^{th}$ order statistic squared

Continuing from the above derivation:

$$
\begin{align}
E(X_{(r)}^2) &= \frac{\gamma_0}{\gamma_1^n} \sum_{k=0}^{n-r}   {n-r \choose k} \gamma_1^k (-1)^{n-r-k}\int_a^c x^2(x-a)^{2n-2k-1}dx \\
&+ \frac{\gamma_0}{\gamma_2^n} (-1)^{(n-r)} \sum_{k=0}^{r-1} {r-1 \choose k} \gamma_2^k \int_c^b x^2(x-b)^{2n-2k-1}dx
\end{align}
$$

$$
\begin{align}
E(X_{(r)}^2) &= \gamma_0 \sum_{k=0}^{n-r}   {n-r \choose k} \gamma_1^{k-n} (-1)^{n-r-k} \frac{(c-a)^{2n-2k}}{2n-2k}\left[c^2 - \frac{2c(c-a)}{2n-2k+1} + \frac{2(c-a)^2}{(2n-2k+1)(2n-2k+2)}\right] \\
&+ \gamma_0 (-1)^{(n-r)} \sum_{k=0}^{r-1} {r-1 \choose k} \gamma_2^{k-n}  \frac{(c-b)^{2n-2k}}{2n-2k}\left[-c^2 + \frac{2c(c-b)}{2n-2k+1}-\frac{2(c-b)^2}{(2n-2k+1)(2n-2k+2)}\right]
\end{align}
$$

$$
\begin{align}
E(X_{(r)}^2) &= r {n \choose r} \sum_{k=0}^{n-r}   {n-r \choose k} (b-a)^{k-n} (c-a)^{n-k} (-1)^{n-r-k} \frac{1}{n-k}\left[c^2 - \frac{2c(c-a)}{2n-2k+1} + \frac{2(c-a)^2}{(2n-2k+1)(2n-2k+2)}\right] \\
&+ r {n \choose r} (-1)^{(n-r)} \sum_{k=0}^{r-1} {r-1 \choose k} (b-a)^{k-n} (c-b)^{n-k} \frac{1}{n-k}\left[-c^2 + \frac{2c(c-b)}{2n-2k+1}-\frac{2(c-b)^2}{(2n-2k+1)(2n-2k+2)}\right]
\end{align}
$$

#### Variance of the $r^{th}$ order statistic

$$V\left(X_{(r)}\right) = E(X_{(r)}^2) - \left[E(X_{(r)})\right]^2$$

#### Numerical Stability of Variance and Expected value of $r^{th}$ order statistic

Although the above derivation gives an exact solution to the variance and expected
value of the $r^{th}$ order statistic of the triangle distribution, the alternative sign
inside the binomial sum and the large orders of magnitude over which those coefficients
span lead to cancellation errors in the results that are not easy to solve.  Exact
solutions can be obtained to arbitrary precision through the use a package like `Rmpfr`.
The `triangle` package defaults to the use of numerical integration for these results.




## Logarithmic Triangle distribution

Define:

$$a_l=\log_{\phi}(a),\ \ b_l=\log_{\phi}(b),\ \ c_l=\log_{\phi}(c),\ \ h=\frac{2}{b_l-a_l}, \ \ \phi = \mbox{log base}$$

$$f(z) = 
\left\{
  \begin{array}{ll}
    \frac{h}{c_l - a_l}(z - a_l) & \mbox{if } a_l \leq z \leq c_l \\
    \frac{h}{c_l - b_l}(z - b_l) & \mbox{if } c_l < z \leq b_l \\
    0 & \mbox{otherwise}
  \end{array}
\right.\ \ \ \ (5)$$

However,

$$E({\phi}^z) \neq {\phi}^{E(z)}\ \ \ \ (6)$$

Therefore, transforming...

$$Y={\phi}^Z$$

$$Z=\log_{\phi}(Y)$$

$$w(y)=\log_{\phi}(y)$$

$$w'(y)=\frac{dz}{dy} = \frac{1}{y\log({\phi})}$$

$$g(y)=f(w(y))w'(y)$$

$$g(y) = 
\left\{
  \begin{array}{ll}
    \frac{2}{(c_l-a_l)(b_l-a_l)\log({\phi})}\frac{log_{\phi}(y) - a_l}{y} & \mbox{if } 0 < a \leq y \leq c \\
    \frac{2}{(c_l-b_l)(b_l-a_l)\log({\phi})}\frac{log_{\phi}(y) - b_l}{y} & \mbox{if } c < y \leq b \\
    0 & \mbox{otherwise}
  \end{array}
\right.\ \ \ \ (7)$$

Define:

$$\beta_1=\frac{2}{(c_l-a_l)(b_l-a_l)}$$

$$\beta_2=\frac{2}{(c_l-b_l)(b_l-a_l)}$$

Finding the CDF, 

$$G(y) = \int_{-\infty}^y g(y)dy$$

$$
\begin{align}
\mbox{for}\ a \leq y \leq c,\ \ G(y) &= \frac{\beta_1}{\log({\phi})} \int_a^y \frac{\log(y)}{y\log({\phi})}-\frac{a_l}{y}dy \\
&=\beta_1 \bigg[\frac{\log_{\phi}^2(y)}{2} - a_l \log_{\phi}(y) - \frac{a_l^2}{2} + a_l^2\bigg]
\end{align}
$$

$$
\begin{align}
\mbox{for}\ c < y \leq b,\ \ G(y) &= G(c) + \frac{\beta_2}{\log({\phi})} \int_c^y \frac{\log(y)}{y\log({\phi})} - \frac{b_l}{y}dy \\
&=G(c) + \beta_2 \bigg[\frac{\log_{\phi}^2(y)}{2} - b_l \log_{\phi}(y) - \frac{c_l^2}{2} + b_l c_l\bigg]
\end{align}
$$

Checking that the CDF is 1 at b,

$$
\begin{align}
G(b) &= \frac{c_l^2 - 2a_l c_l + a_l^2}{(c_l-a_l)(b_l-a_l)} + \frac{-b_l^2-c_l^2+2b_lc_l}{(c_l-b_l)(b_l-a_l)} \\
&= \frac{c_l-a_l}{b_l-a_l} + \frac{-(c_l-b_l)}{b_l-a_l} = 1
\end{align}
$$

Now calculating $E(y)$,

$$
\begin{align}
E(y) &= \int y\ g(y)\ dy \\
&=\frac{\beta_1}{\log({\phi})} \int_a^c \bigg[\frac{\log(y)}{\log({\phi})} - a_l\bigg]dy + 
\frac{\beta_2}{\log({\phi})} \int_c^b \bigg[\frac{\log(y)}{\log({\phi})} - b_l\bigg]dy \\
&=\frac{c\beta_1}{\log^2({\phi})} \bigg[\log(c) - 1 - \log(a) + \frac{a}{c} \bigg] + \frac{c\beta_2}{\log^2({\phi})} \bigg[\frac{-b}{c} - \log(c) + 1 + \log(b) \bigg]
\end{align}
$$

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdfe.R
\name{triangle_cdfe}
\alias{triangle_cdfe}
\title{Triangle parameter estimates using a non-linear fit of the empirical CDF}
\usage{
triangle_cdfe(x, control = stats::nls.control(maxiter = 100, warnOnly = TRUE))
}
\arguments{
\item{x}{the triangle distributed sample}

\item{control}{an object created by \code{stats::nls.control}}
}
\value{
an object of class \code{nls}
}
\description{
Triangle parameter estimates using a non-linear fit of the empirical CDF
}
\examples{
set.seed(10304)
xtest <- rtriangle(100, 1, 5, 2)
cdfe <- triangle_cdfe(xtest)
print(cdfe)
summary(cdfe)
coef(cdfe)
\dontrun{
  confint(cdfe)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mle.R
\name{standard_triangle_mle}
\alias{standard_triangle_mle}
\title{Maximum likelihood estimate of the standard triangle distribution mode}
\usage{
standard_triangle_mle(x, debug = FALSE)
}
\arguments{
\item{x}{sample from a triangle distribution}

\item{debug}{if \code{TRUE} then the function will check the input parameters and print calculation information}
}
\value{
an object of S3 class \code{triangle_mle} containing a list with the call, coefficients,
variance co-variance matrix, minimum negative log likelihood,
number of observations, and the sample
}
\description{
Maximum likelihood estimate of the standard triangle distribution mode
}
\examples{
xtest <- c(0.1, 0.25, 0.3, 0.4, 0.45, 0.6, 0.75, 0.8)
standard_triangle_mle(xtest)

xtest <- rtriangle(20, 0, 1, 0.63)
standard_triangle_mle(xtest)
}
\references{
Samuel Kotz and Johan Rene van Dorp. Beyond Beta \doi{10.1142/5720}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ltriangle.r
\name{ltriangle}
\alias{ltriangle}
\alias{rltriangle}
\alias{dltriangle}
\alias{pltriangle}
\alias{qltriangle}
\title{The Log-Triangle Distribution}
\usage{
rltriangle(
  n = 1,
  a = 1,
  b = 100,
  c = 10^((log10(a) + log10(b))/2),
  logbase = 10
)

dltriangle(x, a = 1, b = 100, c = 10^((log10(a) + log10(b))/2), logbase = 10)

pltriangle(q, a = 1, b = 100, c = 10^((log10(a) + log10(b))/2), logbase = 10)

qltriangle(p, a = 1, b = 100, c = 10^((log10(a) + log10(b))/2), logbase = 10)
}
\arguments{
\item{n}{number of observations. If \code{length(n) > 1}, the length is taken to be the number required.}

\item{a}{lower limit of the distribution.}

\item{b}{upper limit of the distribution.}

\item{c}{mode of the distribution.}

\item{logbase}{the base of the logarithmic scale to use (default to 10)}

\item{x, q}{vector of quantiles.}

\item{p}{vector of probabilities.}
}
\value{
\code{dltriangle} gives the density, \code{pltriangle} gives the
distribution function, \code{qltriangle} gives the quantile function, and
\code{rltraingle} generates random deviates.  Invalid arguments will
result in return value \code{NaN} or \code{NA}.
}
\description{
These functions provide information about the triangle distribution on the
logarithmic interval from \code{a} to \code{b} with a maximum at \code{c}.  \code{dltriangle}
gives the density, \code{pltriangle} gives the distribution function,
\code{qltriangle} gives the quantile function, and \code{rltriangle} generates
\code{n} random deviates.
}
\details{
All probabilities are lower tailed probabilties.  \code{a},
\code{b}, and \code{c} may be appropriate length vectors except in the
case of \code{rtriangle}.
}
\examples{
tri <- rltriangle(100000, 1, 100, 10)
hist(log10(tri), breaks=100, main="Triangle Distribution", xlab="x")
dltriangle(10, 1, 100, 10) # 2/(log10(b)-log10(a)) = 1
qltriangle(pltriangle(10)) # 10
}
\references{
Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988) \emph{The New S Language}.  Wadsworth & Brooks/Cole.
}
\seealso{
\code{\link{.Random.seed}} about random number generation,
\code{\link{runif}}, etc for other distributions.
}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit-plots.R
\name{qqtriangle}
\alias{qqtriangle}
\title{Quantile-Quantile Plot for Triangle Distributed Data}
\usage{
qqtriangle(
  y,
  a,
  b,
  c,
  main = "Triangle Q-Q Plot",
  xlab = "Theoretical Quantiles",
  ylab = "Sample Quantiles",
  ...
)
}
\arguments{
\item{y}{the triangle distributed sample}

\item{a}{the theoretical distribution triangle minimum parameter}

\item{b}{the theoretical distribution triangle maximum parameter}

\item{c}{the theoretical distribution triangle mode parameter}

\item{main}{the plot title}

\item{xlab}{the x-axis label}

\item{ylab}{the y-axis label}

\item{...}{other parameters passed to \code{qqplot}}
}
\value{
a list of x-y coordinates on the plot
}
\description{
Quantile-Quantile Plot for Triangle Distributed Data
}
\examples{
set.seed(10304)
xtest <- rtriangle(100, 1, 5, 2)
theta <- coef(triangle_mle(xtest))
qqtriangle(xtest, theta[1], theta[2], theta[3])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mom.R
\name{triangle_mom}
\alias{triangle_mom}
\title{Triangle distribution method of moments estimate}
\usage{
triangle_mom(x, na.rm = FALSE, type = 1)
}
\arguments{
\item{x}{triangle distribution sample}

\item{na.rm}{whether to remove NA samples.  Default is FALSE.}

\item{type}{the type of method of moments.  Type 1 uses the min and max.  Type2
minimizes the distance from the calculated mean, variance, and skewness to the
sample mean, variance, and skewness}
}
\value{
a named vector of the parameter estimates
}
\description{
Triangle distribution method of moments estimate
}
\examples{
set.seed(1204)
x <- rtriangle(20, 0, 2, 1.5)
triangle_mom(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mle.R
\name{triangle_mle}
\alias{triangle_mle}
\title{Maximum likelihood estimate of the triangle distribution parameters}
\usage{
triangle_mle(x, debug = FALSE, maxiter = 100, boot_var = FALSE, boot_rep = 500)
}
\arguments{
\item{x}{sample from a triangle distribution}

\item{debug}{if \code{TRUE} then the function will check the input parameters and print calculation information}

\item{maxiter}{the maximum number of cycles of optimization between maximizing \code{a} and \code{b} given \code{c}
and maximizing \code{c} given \code{a} and \code{b}}

\item{boot_var}{should the variance be computed with a bootstrap sample?}

\item{boot_rep}{The number of bootstrap replications}
}
\value{
an object of S3 class \code{triangle_mle} containing a list with the call, coefficients,
variance co-variance matrix, minimum negative log likelihood, details of the optimization
number of observations, and the sample
}
\description{
Maximum likelihood estimate of the triangle distribution parameters
}
\examples{
xtest <- c(0.1, 0.25, 0.3, 0.4, 0.45, 0.6, 0.75, 0.8)
triangle_mle(xtest)

xtest <- rtriangle(20, 1, 5, 3.5)
triangle_mle(xtest)
}
\references{
Samuel Kotz and Johan Rene van Dorp. Beyond Beta \doi{10.1142/5720}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dtriangle.R, R/ptriangle.r, R/qtriangle.R,
%   R/rtriangle.r
\name{triangle}
\alias{triangle}
\alias{dtriangle}
\alias{ptriangle}
\alias{qtriangle}
\alias{rtriangle}
\title{The Triangle Distribution}
\usage{
dtriangle(x, a = 0, b = 1, c = (a + b)/2)

ptriangle(q, a = 0, b = 1, c = (a + b)/2)

qtriangle(p, a = 0, b = 1, c = (a + b)/2)

rtriangle(n = 1, a = 0, b = 1, c = (a + b)/2)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{a}{lower limit of the distribution.}

\item{b}{upper limit of the distribution.}

\item{c}{mode of the distribution.}

\item{p}{vector of probabilities.}

\item{n}{number of observations. If \code{length(n) > 1}, the length is taken to be the number required.}
}
\value{
\code{dtriangle} gives the density, \code{ptriangle} gives the
distribution function, \code{qtriangle} gives the quantile function, and
\code{rtriangle} generates random deviates.  Invalid arguments will result
in return value \code{NaN} or \code{NA}.
}
\description{
These functions provide information about the triangle
distribution on the interval from \code{a} to \code{b} with a maximum at
\code{c}.  \code{dtriangle} gives the density, \code{ptriangle} gives the
distribution function, \code{qtriangle} gives the quantile function, and
\code{rtriangle} generates \code{n} random deviates.
}
\details{
All probabilities are lower tailed probabilities.
\code{a}, \code{b}, and \code{c} may be appropriate length vectors except in
the case of \code{rtriangle}.  \code{rtriangle} is derived from a draw from
\code{runif}. The triangle distribution has density:
\deqn{f(x) = \frac{2(x-a)}{(b-a)(c-a)}}{f(x) = 2(x-a) / [(b-a)(c-a)]}
for \eqn{a \le x < c}{a <= x < c}.
\deqn{f(x) = \frac{2(b-x)}{(b-a)(b-c)}}{f(x) = 2(b-x) / [(b-a)(b-c)]}
for \eqn{c \le x \le b}{c <= x <= b}.
\eqn{f(x) = 0} elsewhere.
The mean and variance are:
\deqn{E(x) = \frac{(a + b + c)}{3}}{E(x) = (a + b + c) / 3}
\deqn{V(x) = \frac{1}{18}(a^2 + b^2 + c^2 - ab - ac - bc)}{V(x) = (a^2 + b^2 + c^2 - ab - ac - bc) / 18}
}
\examples{
## view the distribution
tri <- rtriangle(100000, 1, 5, 3)
hist(tri, breaks=100, main="Triangle Distribution", xlab="x")
mean(tri) # 1/3*(1 + 5 + 3) = 3
var(tri)  # 1/18*(1^2 + 3^2 + 5^2 - 1*5 - 1*3 - 5*3) = 0.666667
dtriangle(0.5, 0, 1, 0.5) # 2/(b-a) = 2
qtriangle(ptriangle(0.7)) # 0.7
}
\references{
Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988) \emph{The New S Language}.  Wadsworth & Brooks/Cole.
}
\seealso{
\code{\link{.Random.seed}} about random number generation,
\code{\link{runif}}, etc for other distributions.
}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit-plots.R
\name{compare_triangle_fit}
\alias{compare_triangle_fit}
\title{Compare multiple triangle distributions fits}
\usage{
compare_triangle_fit(
  y,
  cols = c("red", "blue", "green"),
  main = "Triangle Fit Comparison",
  ...
)
}
\arguments{
\item{y}{the triangle distributed sample}

\item{cols}{the colors of the CDF-based estimates, the maximum likelihood estimates, and the method of moments estimates}

\item{main}{the plot title}

\item{...}{other parameters passed to \code{plot.ecdf}}
}
\description{
Compare multiple triangle distributions fits
}
\examples{
set.seed(10304)
xtest <- rtriangle(100, 1, 5, 2)
compare_triangle_fit(xtest)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mle-utils.R
\name{summary.triangle_mle}
\alias{summary.triangle_mle}
\alias{print.triangle_mle}
\alias{coef.triangle_mle}
\alias{logLik.triangle_mle}
\alias{AIC.triangle_mle}
\alias{BIC.triangle_mle}
\alias{vcov.triangle_mle}
\alias{profile.triangle_mle}
\alias{confint.triangle_mle}
\title{Utility Methods for S3 class triangle_mle}
\usage{
\method{summary}{triangle_mle}(object, ...)

\method{print}{triangle_mle}(x, ...)

\method{coef}{triangle_mle}(object, ...)

\method{logLik}{triangle_mle}(object, ...)

\method{AIC}{triangle_mle}(object, ..., k = 2)

\method{BIC}{triangle_mle}(object, ...)

\method{vcov}{triangle_mle}(object, ...)

\method{profile}{triangle_mle}(fitted, ...)

\method{confint}{triangle_mle}(object, parm, level = 0.95, ...)
}
\arguments{
\item{object}{class triangle_mle from a call to \code{triangle_mle()}}

\item{...}{not used except for \code{print} (other arguments passed to \code{printCoefmat})}

\item{x}{the \code{triangle_mle} object}

\item{k}{the penalty per parameter to be used; the default \code{k = 2}}

\item{fitted}{an object of class triangle_mle}

\item{parm}{parameters to be given confidence intervals passed to \code{stats4::confint}}

\item{level}{confidence interval level passed to \code{stats4::confint}}
}
\value{
an object of class summary.mle

\code{print.triangle_mle}: \code{x} invisibly

\code{coef.triangle_mle}: a vector of coefficients

\code{logLik.triangle_mle}: an object of class \code{logLik}

\code{AIC.triangle_mle}: the AIC

\code{BIC.triangle_mle}: the BIC

\code{vcov.triangle_mle}: the variance co-variance matrix

\code{profile.triangle_mle}: an object of class \code{profile.mle}

\code{confint.triangle_mle}: a matrix of parameter confidence intervals
}
\description{
Utility Methods for S3 class triangle_mle
}
\examples{
set.seed(1234)
x <- rtriangle(100, 0, 1, 0.5)
mle1 <- triangle_mle(x)
summary(mle1)
print(mle1)
coef(mle1)
logLik(mle1)
AIC(mle1)
BIC(mle1)
vcov(mle1)
\dontrun{
  prof <- profile(mle1)
  stats4::plot(prof)
  confint(mle1, 1:3, level = 0.95)
}
}
